#ifndef COMMON_HPP
#define COMMON_HPP

#include "config.h"

#include <string>
#include <list>
#include <vector>
#include <fcntl.h>
#include <errno.h>
//mypair
typedef std::pair<std::string, std::string> MyPair;
const MyPair emptyMyPair = MyPair("","");

MyPair check_type(const std::string& s, const std::list<MyPair>& temp);

namespace conv {
  int atoi(const std::string& s);
  std::string itos(int i);
  float atof(const std::string& s);
  std::string ftos(float f);
  bool stob(const std::string& s);
  std::string btos(bool b);
}

namespace string_format {

  std::string unique_folder_name(const std::string& folder);

  void format_to_size(std::string& s, const std::string& font, int size, bool dots, bool long_string = false);

  // if a string is longer than maxlen, where maxlen is in number of chars,
  // pps will transform the string into: the start, ..., the end of the string
  void pretty_print_string(std::string& s, int maxlen);

  std::string pretty_print_string_copy(const std::string& s, int maxlen);

  std::string pretty_pad_string(const std::string& s, int maxlen, const char& padchar);

  // escapes ' in a string
  std::string escape_db_string(const std::string& input);
  std::string escape_string(const std::string& input);

  // convertes // to /, useful for path prettyprinting
  std::string remove_doubles(const std::string& doubles);

  std::string lowercase(std::string s);

  // trims spaces, tabs and newline off the start and end of a string
  std::string trim(std::string s);

  int calculate_string_width(const std::string& text, const std::string& font);
  // returns a pair of w, h
  std::pair<int, int> calculate_string_size(const std::string& text, const std::string& font);

  // please note that get_line must not contain a newline
  std::string get_line(const std::string& text, int &pos, int width, const std::string& font);

  // return the difference in the number of chars between two strings
  int difference(const std::string& lhs, const std::string& rhs);

  // returns the string with as many letters in common from input strings
  // beginning from the start of the string and ending when the first letter
  // doesn't match
  std::string greatest_common_substring(const std::string& lhs, const std::string& rhs);

  // convert a string in local format to UTF-8
  std::string convert(const std::string& in, bool mp3_locale = false);

  //wrap a line of text  and returns a vector of strings, each one being a line whose size
  //fits in maxwidth
  int wrap_strings(std::string& s, const std::string& font, int maxwidth, std::vector<std::string>& v);
}

// rounds to nearest int
const int round_to_int(double number);

const int seconds_to_minutes(int time);
// returns time % 60 (the seconds left over)
const int remaining_seconds(int time);

// can also check if a dir exists
bool file_exists(const std::string& filename);
bool dir_exists(const std::string& dir);
bool havemydir(const std::string &subdir, std::string *retpath=NULL);
bool recurse_mkdir(std::string homedir, std::string subdir, std::string *retpath);
// this function is recursive, remember the last slash
bool abs_recurse_mkdir(const std::string& dir, std::string *retpath=NULL);
void create_if_not_exists(const std::string& dir);

//string safe version of getenv()
std::string s_getenv(const std::string& envname);

//string safe version of strerror();
std::string s_strerror(int);
std::string s_strerror();

template<typename T>
inline T vector_lookup(const std::vector<T>& v, unsigned int pos)
{
#ifdef gcc_ver_major_3
  return v.at(pos);
#endif
#ifdef gcc_ver_major_2
  return v[pos];
#endif
}

void clean_up(int signal);
void deinit_mms(bool from_cleanup);

class rect
{
public:
  int x, y, w, h;
  rect()
  {}
  rect(int x_, int y_, int w_, int h_)
    : x(x_), y(y_), w(w_), h(h_)
  {}

  int area()
  { return w*h; }

  bool operator > (const rect& r)
  { return x < r.x && y < r.y && x + w >= r.x + r.w && y + h >= r.y + r.h; }

  bool operator >= (const rect& r)
  { return x <= r.x && y <= r.y && x + w >= r.x + r.w && y + h >= r.y + r.h; }

  bool intersects(int x_, int y_)
  { return x <= x_ && x + w >= x_ && y <= y_ && y + h >= y_; }

  std::string to_string() const
  {
    return (conv::itos(x) + ":" + conv::itos(y) + ":" + conv::itos(w) + ":" + conv::itos(h));
  }
};

namespace filemutex {
#define  mutexname "/tmp/mms.lock"
  int lock();
}

#endif
