#include "config.h"

#include "config.hpp"
#include "common.hpp"

#include "lyrics.hpp"

#include "common.hpp"
#include "gettext.hpp"
#include "graphics.hpp"
#include "updater.hpp"
#include "input.hpp"

#include "audio.hpp"
#include "startmenu.hpp"

#include "tinyxml.h"
#include "tinystr.h"

#include "wget_wrapper.hpp"

// saving
#include <fstream>
#include <iostream>

#include <cc++/url.h>

using std::string;

pthread_mutex_t Lyrics::singleton_mutex = PTHREAD_MUTEX_INITIALIZER;

void LyricsFetch::run()
{
  S_Lyrics::get_instance()->check_status();
}

void LyricsLoop::run()
{
  Lyrics *lyrics = S_Lyrics::get_instance();
  while(true) {
    lyrics->update_px_count();
    sleep(lyrics->get_period()); // ms
  }
}

Lyrics::Lyrics()
  : enabled(false), running(false)
{
  vecLyric.clear();

  render = S_Render::get_instance();

  /*
  Updater &updater = Updater::get_instance();
  TimeFunctori<Lyrics> *time_func = new TimeFunctori<Lyrics>(this, &Lyrics::idle_status);
  TimeFunctorv<Lyrics> *call_func = new TimeFunctorv<Lyrics>(this, &Lyrics::check_status);
  updater.timer.add(TimeElement("fetchlyric", time_func, call_func));
  */

  tmp_location = conf->p_temp_image_loc() + "/wget-temp-lyric";

  if (!recurse_mkdir(conf->p_var_data_dir(),"lyrics/", &lyrics_dir)){
      DebugPrint perror(dgettext("mms-audio", "Could not create directory ") + lyrics_dir,
			Print::DEBUGGING, DebugPrint::CRITICAL, "LYRICS");
      //FIXME: shouldn't this be critical???
  }

  std::pair<int, int> size =  string_format::calculate_string_size("(asdfghjkjlASDFGHJKL):", "Vera/22");

  font_size = size.second;

  LyricsLoop *ll = new LyricsLoop;
  ll->start();

}

int Lyrics::idle_status()
{
  return 500; // ms
}

void Lyrics::check_status()
{
  if (enabled) {
    std::cout << "Lyrics::check_status()" << std::endl;
    Load() || search_result();
    if (buff_text.size() && !vecLyric.size())
      parse_text(buff_text);
  }
  enabled = false;
}

string Lyrics::mainloop()
{
  return "";
}

bool Lyrics::find_lyric(string artist, string title)
{

  if (cache_artist == artist && cache_title == title)
    return true;

  cache_artist = artist;
  cache_title = title;
  vecLyric.clear();
  buff_text = "";
  px_count = 0;
  tm_old = 0;
  total_time = 0;
  current_time = 0;
  display_height = 0;
  ln_displace = 0;
  period_fix = 0;
  enabled = true;

  LyricsFetch *lf = new LyricsFetch;
  lf->start();

  return true;
}

bool Lyrics::Load()
{
  const int BUFFER_SIZE = 2048;
  std::ifstream file;

  file.open(filename().c_str());
  if (file.is_open())
    {
      char tmpBuff[BUFFER_SIZE];
      while(!file.eof()) {
	file.getline(tmpBuff,BUFFER_SIZE -1);
	buff_text += tmpBuff;
	buff_text += "\n";
      }
      file.close();
      std::cout << "finding on cache" << std::endl;
      return true;
    }
  return false;
}

void Lyrics::Save()
{
  std::cout << "Lyrics::Save()" << std::endl;
  if (buff_text.size())
    parse_text(buff_text);
  if (!vecLyric.size())
    return;

  std::ofstream file;
  string curLine;
  std::vector<std::string>::iterator itVecLyr;

  file.open(filename().c_str());
  if (file.is_open())
  {

    for(itVecLyr = vecLyric.begin(); itVecLyr != vecLyric.end(); itVecLyr++) {
      curLine = *(itVecLyr);
      file << curLine << std::endl;
    }
    file.close();
    std::cout << "save on cache" << std::endl;
  }
}

std::string Lyrics::filename()
{
   return (lyrics_dir + string_format::lowercase(string_format::convert(cache_artist)) + " - " + string_format::lowercase(string_format::convert(cache_title)));
}

bool Lyrics::search_result()
{
  std::cout << "finding on inet" << std::endl;
  string url = "http://api.leoslyrics.com/api_search.php?auth=MMS&artist="
	+ string_format::lowercase(string_format::convert(cache_artist))
	+ "&songtitle="
	+ string_format::lowercase(string_format::convert(cache_title));

  std::string searchxml, hid = "";
  int counter = 0;
  //list<pair<string, string> > genrelist;

  if (WgetWrapper::download(url, searchxml) > 0)
  {
    TiXmlDocument searchxmldoc;
    int startpos = 0;
    if((startpos = searchxml.find("<?xml")) != string::npos)
    {
      if(searchxmldoc.Parse(searchxml.substr(startpos).c_str(), 0 , TIXML_ENCODING_UTF8))
      {
	TiXmlElement* root_element = searchxmldoc.RootElement();
	TiXmlElement* element;

	if (root_element == 0)
	  return false;

	for (element = root_element->FirstChildElement("searchResults"); element != 0;
	     element = element->NextSiblingElement("searchResults")) {
	  element= element->FirstChildElement("result");
	  if (element != 0)
	    hid = element->Attribute("hid");
	  else
	    break;
	}

	std::cout << "hid :" << hid << std::endl;

/*        TiXmlElement* element;
    	element = searchxmldoc.FirstChildElement();
    	hid = element->Attribute("hid");
  	std::cout << "hid :" << hid << std::endl;*/

	if (!hid.empty())
	  return get_text(hid);
      }
    }
  }
  std::cout << "fail !!! " << std::endl;
  return false;
}

bool Lyrics::get_text(const string& hid)
{
  std::cout << "Lyrics::get_text(" << hid << ")" << std::endl;
  char hid_encoded[100];

  ost::urlEncode(hid.c_str(), hid_encoded, sizeof(hid_encoded));
  string url = "http://api.leoslyrics.com/api_lyrics.php?auth=MMS&hid=" + string(hid_encoded);


std::string searchxml;

int ret;

const char* text;

  //list<pair<string, string> > genrelist;

  if (WgetWrapper::download(url, searchxml) > 0)
  {
    TiXmlDocument searchxmldoc;
    int startpos = 0;
    if((startpos = searchxml.find("<?xml")) != string::npos)
    {
      //std::cout << "OPEN XML =" << searchxml << std::endl;
      if(searchxmldoc.Parse(searchxml.substr(startpos).c_str(), 0 , TIXML_ENCODING_UTF8))
      {
	TiXmlNode* root_element = searchxmldoc.RootElement();
	TiXmlNode* element;
	TiXmlNode* response;

	if (root_element == 0)
    	  return false;

	response = root_element->FirstChildElement("response");
	ret = conv::atoi(response->ToElement()->Attribute("code"));

	if (ret == 0) {
	  for (element = root_element->FirstChildElement("lyric"); element != 0;
          element = element->NextSiblingElement("lyric")) {
	    element = element->FirstChildElement("text");
	    element = element->FirstChild();
	    buff_text = string_format::trim(element->Value());
            Save();
            std::cout << "ending Lyrics::get_text(" << hid << ")" << std::endl;
            return true;
          }
        }
      }
    }
  }
  return false;
}

bool Lyrics::parse_text(const string& text, string nsec)
{
  int startpos = -1;
  int currentpos = 0;
  int w, h;
  string line, temp, buff;
  temp = text;

  vector<std::string> vecTmp;
  vecTmp.clear();

  /*for (int i=0; i < 14; i++)
	 vecTmp.push_back(" ");
*/
  std::string::size_type pos = temp.find(nsec);
  if (pos == std::string::npos)
	return parse_text(text, "\n");

  while (pos != std::string::npos) {
    buff = string_format::trim(text.substr(startpos + 1 , pos - startpos - 1 ));
    vecTmp.push_back(buff);
    temp.replace(pos, 1, "|");
    startpos = pos;
    pos = temp.find(nsec);
  }
    buff = string_format::trim(text.substr(startpos + 1 , pos - startpos - 1 ));
    vecTmp.push_back(buff);

  /*for (int i=0; i < 7; i++)
	 vecTmp.push_back(" "); */

  vecLyric = vecTmp;
  std::cout << "Lyrics::parse_text(text) loaded "<< vecLyric.size() << " lines" << std::endl;
  return true;
}


bool Lyrics::is_enabled()
{
  return (vecLyric.size() ? true : false);
}

void Lyrics::set_displace(int step)
{
  int nvalue = ln_displace + step;
  if (std::abs(nvalue) < vecLyric.size() - display_lines())
    ln_displace += step;
}

int Lyrics::get_displace()
{
  return ln_displace;
}

void Lyrics::set_total_time(int time)
{
  total_time = time;
}

void Lyrics::set_current_time(int time)
{
  current_time = time;

  if (total_time != 0) {
    int tm = current_time * 100 / total_time;
    int df = tm - tm_old;
    // resync lyric with track time when rewind or fastforward
    if (std::abs(df) > 2) {
      px_count = current_time * (vecLyric.size() * font_size - display_height) / total_time;
    }
    tm_old = tm;
    // end of count
    //update_px_count();
  } else
    px_count = 0;
}

void Lyrics::update_px_count()
{
  if (running && get_line() + ln_displace <=  max_line())
    px_count++;
}

void Lyrics::play()
{
  running = true;
}

void Lyrics::pause()
{
  running = false;
}


void Lyrics::set_display_height(int h)
{
  display_height = h;
}

float Lyrics::get_frequency()
{
  float pixel = float(vecLyric.size()) * float(font_size) - display_height;
  float freq  = total_time ? pixel / total_time : 0;
  return freq;
}

float Lyrics::get_period()
{
  float freq = get_frequency();
  float per = (freq ?  1 / freq * 1000 : 0);
  per -=  period_fix;
  period_fix = per - int(per);
  return (per ? per : 250); // ms
}

/**
  *
  * return int numbers of lines to display
  */
int Lyrics::display_lines() {
  return int(display_height / font_size);
}

/**
  *
  * return int numbers of lines to display
  *
  */
int Lyrics::max_line()
{
  return vecLyric.size() - display_lines();
}

int Lyrics::get_line()
{
  int val = px_count / font_size + ln_displace;

  if (val > 0 && val < vecLyric.size())
    return val;
  else
    return 0;
}


int Lyrics::get_anim()
{
  return (px_count / font_size + ln_displace) >= 0 ? px_count % font_size : 0;
}
