
#include "clock.hpp"

#include "image.hpp"
#include "renderer.hpp"
#include "gettext.hpp"
#include "graphics.hpp"
#include "notify_area.hpp"

#include <cc++/numbers.h>
#include <string>

#include "boost.hpp"

using std::string;

pthread_mutex_t Clock::singleton_mutex = PTHREAD_MUTEX_INITIALIZER;

Clock::Clock()
{
#ifdef use_nls
  // gettext
  setlocale(LC_ALL, ""); /* set from the environment variables */
  bindtextdomain("mms-clock", mms_prefix"/share/locale");
  bind_textdomain_codeset("mms-clock", nl_langinfo(CODESET));
#endif
  nArea->add(NotifyElement("Clock", boost::bind(&Clock::ShowNotifyArea, this))); 
}

void Clock::ShowNotifyArea() 
{ 
  ost::Date date;
  ost::Time time;

  string strMsg;
  std::pair<int, int> size;
  int ow, oh;  

  strMsg = doubleZero(time.getHour()) + ":" + doubleZero(time.getMinute());
  size = string_format::calculate_string_size(strMsg, "Vera/28");
  nArea->status_overlay.add(new TObj(strMsg, "Vera/28", 
			conf->p_h_res() - NotifyArea::width/2 - size.first/3,
			conf->p_v_res() - NotifyArea::height/4 - size.second, 
			themes->notify_area_font1, 
			themes->notify_area_font2, 
			themes->notify_area_font3, 2));

  
  strMsg = doubleZero(date.getDay());
  size =  string_format::calculate_string_size(strMsg, "Vera/18");
  oh = conf->p_v_res() - size.second;
  ow = conf->p_h_res() - size.first + 5;
  nArea->status_overlay.add(new TObj(strMsg, "Vera/18", 
			ow , oh, 
			themes->notify_area_font1, 
			themes->notify_area_font2, 
			themes->notify_area_font3, 2));



  strMsg = dayOfWeek(date.getDayOfWeek());
  size = string_format::calculate_string_size(strMsg, "Vera/14");
  ow -= size.first - 5;
  nArea->status_overlay.add(new TObj(strMsg, "Vera/14", 
			ow, oh, 
			themes->notify_area_font1, 
			themes->notify_area_font2, 
			themes->notify_area_font3, 2));
}

string Clock::doubleZero(int time)
{
  char buffer [512];
  snprintf(buffer, 512, "%02d", time);
  return buffer;
}

string Clock::dayOfWeek(int date)
{
  string buff;
  switch (date) {
	case 0: buff = dgettext("mms-clock","sunday"); break;
	case 1: buff = dgettext("mms-clock","monday"); break;
	case 2: buff = dgettext("mms-clock","tuesday"); break;
	case 3: buff = dgettext("mms-clock","wednesday"); break;
	case 4: buff = dgettext("mms-clock","thursday"); break;
	case 5: buff = dgettext("mms-clock","friday"); break;
	case 6: buff = dgettext("mms-clock","saturday"); break;
  }
  return buff;
}

