#include "epg_pic.hpp"

#include "graphics.hpp"

#include "boost.hpp"

using std::list;
using std::string;

EpgPic::EpgPic()
  : Epg(78)
{}

void EpgPic::print_timelines(double diff)
{
  if (diff > 0 && diff < timeslots)
    render->current.add(new RObj(int(x_start + (diff*30*60)/scale_factor), y_start, 2, rows*40+10,
				 themes->epg_time_bar1, themes->epg_time_bar2,
				 themes->epg_time_bar3, themes->epg_time_bar4, 1));

  render->current.add(new RObj(x_start, y_start, 2, rows*40+10,
			       themes->epg_bar1, themes->epg_bar2,
			       themes->epg_bar3, themes->epg_bar4, 1));

  for (int x = 0; x < timeslots; ++x) {
    time_t tmp_time = cur_time + x*30*60;
    char time_str[30];
    strftime(time_str, 30, "%H:%M", localtime(&tmp_time));
    render->current.add(new TObj(time_str, "Vera/17",
				 static_cast<int>(x_start-28 + (x*30*60)/scale_factor), y_start-30,
				 themes->epg_font1, themes->epg_font2,
				 themes->epg_font3, 2));
    render->current.add(new RObj(static_cast<int>(x_start + ((x+1)*30*60)/scale_factor), y_start, 2, rows*40+10,
				 themes->epg_bar1, themes->epg_bar2, themes->epg_bar3,
				 themes->epg_bar4, 1));
  }
}

void EpgPic::print_channel(int y, std::list<Channel>::iterator cur_chan)
{
  render->current.add(new PObj(cur_chan->logo, 15, y*40+y_start+10, 2, NOSCALING, false));
}

void EpgPic::print_desc_channel(const std::string& orig_title)
{
  render->current.add(new PObj(sel_channel->logo, 75, 100, 2, NOSCALING, false));

  string title = orig_title;
  string_format::format_to_size(title, "Vera/22", conf->p_h_res()-(145 + 70), true);

  render->current.add(new TObj(title, "Vera/22", 140, 95,
			       themes->epg_font1, themes->epg_font2, themes->epg_font3, 2));
}

bool EpgPic::check_tv_file()
{
  // check if the icons can be found
  bool icons = true;

  foreach (Channel& channel, channels) {

    string::size_type dot_pos = channel.id.find(".");

    if (dot_pos == string::npos) {
      dot_pos = channel.id.size();
      print_critical(dgettext("mms-epg", "Error in channel id, please report this problem"), "EPG");
    }

    channel.logo = conf->p_homedir() + "epg_icons/"
      + channel.id.substr(0, channel.id.find(".")) + ".gif";
    if (!file_exists(channel.logo)) {
      print_critical(dgettext("mms-epg", "Failed to find icon ") + channel.logo, "EPG");
      icons = false;
      break;
    }
  }

  if (!icons) {
    Print pdialog(Print::SCREEN);
    pdialog.add_line(dgettext("mms-epg", "Could not find icons for epg"));
    pdialog.add_line("");
    pdialog.add_line(dgettext("mms-epg", "Please install the icons and try again"));
    pdialog.print();

    return false;
  } else
    return Epg::check_tv_file();
}
