#include "plugin.hpp"
#include "epg.hpp"
#include "themes.hpp"
#include "print.hpp"
#include "cd.hpp"
#include "config.hpp"
#include "common-feature.hpp"

#include "epg_pic.hpp"
#include "epg_text.hpp"

void view_epg()
{
  Epg *epg = get_class<Epg>(dgettext("mms-epg", "Electronic Program Guide"));

  if (!epg->loaded_correctly) {
    epg->read_db();
    epg->loaded_correctly = true;
  }

  if (epg->reload_needed) {
    DialogWaitPrint pdialog(dgettext("mms-epg", "Rereading TV listings file"), 1000);
    epg->read_db();
  }

  if (!epg->check_tv_file())
    epg->reload_needed = true;
  else {
    epg->reload_needed = false;
    epg->mainloop();
  }
}

class EpgPlugin : public FeaturePlugin
{
public:
  EpgPlugin() {
    Config *conf = S_Config::get_instance();
    EPGConfig *epg_conf = S_EPGConfig::get_instance();

    epg_conf->parse_configuration_file(conf->p_homedir());

    if (epg_conf->p_picture_epg())
      module = new EpgPic();
    else
      module = new EpgText();

    Themes *themes = S_Themes::get_instance();
    features.push_back(startmenu_item(dgettext("mms-epg", "View Electronic Program Guide"),
				      "epg",themes->startmenu_epg, 0, &view_epg));
  }
  std::string plugin_name() const { return dgettext("mms-epg", "Electronic Program Guide"); }
  int priority() const { return 4; }
  unsigned long capability() const { return 0; }
};

MMS_PLUGIN(EpgPlugin)
