#include "epg_text.hpp"

#include "graphics.hpp"

#include "boost.hpp"

using std::string;
using std::list;

EpgText::EpgText()
  : Epg(int(78+30*60/10.25-25))
{}

void EpgText::print_timelines(double diff)
{
  if (diff > 0 && diff < timeslots)
    render->current.add(new RObj(int(x_start + diff*30*60/scale_factor), y_start-10, 2, rows*40+18,
				 themes->epg_time_bar1, themes->epg_time_bar2,
				 themes->epg_time_bar3, themes->epg_time_bar4, 1));

  render->current.add(new RObj(x_start, y_start-10, 2, rows*40+18,
			       themes->epg_bar1, themes->epg_bar2,
			       themes->epg_bar3, themes->epg_bar4, 1));

  for (int x = 0; x < timeslots; ++x) {
    time_t tmp_time = cur_time + x*30*60;
    char time_str[30];
    strftime(time_str, 30, "%H:%M", localtime(&tmp_time));
    render->current.add(new TObj(time_str, "Vera/17",
				 static_cast<int>(x_start-28 + (x*30*60)/scale_factor), y_start-40,
				 themes->epg_font1, themes->epg_font2,
				 themes->epg_font3, 2));
    render->current.add(new RObj(static_cast<int>(x_start + ((x+1)*30*60)/scale_factor), y_start-10, 2, rows*40+18,
				 themes->epg_bar1, themes->epg_bar2, themes->epg_bar3,
				 themes->epg_bar4, 1));
  }
}

void EpgText::print_desc_channel(const std::string& orig_title)
{
  string title = orig_title;
  string_format::format_to_size(title, "Vera/22", conf->p_h_res()-(2*75), true);

  render->current.add(new TObj(title, "Vera/22", 75, 95,
			       themes->epg_font1, themes->epg_font2, themes->epg_font3, 2));
}

void EpgText::print_channel(int y, std::list<Channel>::iterator cur_chan)
{
  if (!cur_chan->logo.empty())
    render->current.add(new PObj(cur_chan->logo, 15, y*40+y_start+10, 2, NOSCALING, false));
  else {
    string channel_name = cur_chan->display_name;
    string_format::format_to_size(channel_name, "Vera/16", x_start-10, false);
    render->current.add(new TObj(channel_name, "Vera/16", 15, y*40+y_start,
				 themes->epg_font1, themes->epg_font2,
				 themes->epg_font3, 2));
  }
}

bool EpgText::check_tv_file()
{
  foreach (Channel& channel, channels) {

    string::size_type dot_pos = channel.id.find(".");

    if (dot_pos == string::npos) {
      dot_pos = channel.id.size();
      print_critical(dgettext("mms-epg", "Error in channel id, please report this problem"), "EPG");
    }

    string logo_file = conf->p_homedir() + "epg_icons/" + channel.id.substr(0, channel.id.find(".")) + ".gif";
    if (file_exists(logo_file))
      channel.logo = logo_file;
    else
      print_debug(dgettext("mms-epg", "logo not found:") + logo_file, "EPG");
  }

  return Epg::check_tv_file();
}

