// used for #ifdef
#include "config.h"

#include "libfspp.hpp"

#include "pictures.hpp"

//Input
#include "input.hpp"

#include "config.hpp"
#include "common.hpp"
#include "graphics.hpp"
#include "updater.hpp"
#include "common-feature.hpp"
#include "gettext.hpp"
#include "rand.hpp"

#include "search.hpp"

#include "startmenu.hpp"
#include "busy_indicator.hpp"
#include "shutdown.hpp"
#include "extra_menu.hpp"
#include "touch.hpp"
#include "SSaverobj.h"

// ostringstream
#include <sstream>

#include <iostream>

#include <unistd.h>

#ifdef use_nls
#include <langinfo.h>
#include <locale.h>
#endif

using std::list;
using std::vector;
using std::string;

Pictures::Pictures()
  : db((conf->p_var_data_dir() + "pictures.db").c_str()),
    exit_loop(false), in_fullscreen(false), orientation(0),
    zoom_level(1), zoom_position(1), reload_dirs(false),
    thumbnailed_all_files(false), pause(false), navigating_media(false)
{
  pic_conf = S_PictureConfig::get_instance();

  set_folders();

  reset();

  y_start = 70;

  image_width = 125;
  image_height = 96;

  if (conf->p_v_res() == 405) {
    image_width = 80;
    image_height = 67;
  }

  // text and space around image
  image_width_all = image_width+25;
  image_height_all = image_height+55;
  image_height_all_eks_text = image_height + 6;

  images_per_row = (conf->p_h_res()-2*25)/image_width_all;
  // lets make some more room for text
  image_width_all += ((conf->p_h_res()-2*25)%image_width_all)/images_per_row;

  rows = (conf->p_v_res()-2*y_start)/image_height_all;
  rows_search = (conf->p_v_res()-2*y_start-20)/image_height_all;

  // otherwise navigation won't work
  assert(rows > 1);
  assert(rows_search > 1);

  int to_spread_search = (conf->p_v_res()-2*y_start-20)-rows_search*image_height_all;
  image_height_all_search = to_spread_search/(rows_search+1) + image_height_all;

  int to_spread = (conf->p_v_res()-2*y_start)-rows*image_height_all;
  image_height_all += to_spread/(rows+1);
  y_start += to_spread/(rows+1);

  check_db();
}

Pictures::~Pictures()
{
  db_mutex.enterMutex();
  db.close();
  db_mutex.leaveMutex();
}

void Pictures::reset()
{
  for (int i = 0, size = folders.size(); i < size; ++i)
    folders.pop();

  folders.push(std::make_pair(picture_folders, 0));
}

void Pictures::set_folders()
{
  list<string> picture_dirs = pic_conf->p_picture_dirs(); // make compiler happy
  // make sure picture folders are not malformed
  foreach (string& dir, picture_dirs)
    if (dir[dir.size()-1] != '/')
      picture_folders.push_back(dir + '/');
    else
      picture_folders.push_back(dir);

#ifdef use_inotify
  S_Notify::get_instance()->register_plugin("pictures", picture_folders,
					    boost::bind(&Pictures::fs_change, this, _1, _2));
#endif

  list<string> picture_dirs_no_watch = pic_conf->p_picture_dirs_no_watch(); // make compiler happy
  // make sure picture folders are malformed
  foreach (string& dir, picture_dirs_no_watch)
    if (dir[dir.size()-1] != '/')
      picture_folders.push_back(dir + '/');
    else
      picture_folders.push_back(dir);
}

void Pictures::startup_updater()
{
  BackgroundUpdater *background_updater = S_BackgroundUpdater::get_instance();
  background_updater->timer.add(TimeElement("extra_thumbnails", boost::bind(&Pictures::check_thumbnail, this),
					    boost::bind(&Pictures::background_thumbnails, this)));

  S_Search::get_instance()->register_module(SearchModule(dgettext("mms-pictures", "Pictures"), boost::bind(&Pictures::gen_search_list, this, _1),
							 boost::bind(&Pictures::reenter, this, _1)));
}

Pictures::PicOpts::PicOpts()
{
#ifdef use_nls
  // gettext
  setlocale(LC_ALL, ""); /* set from the environment variables */
  bindtextdomain("mms-pictures", mms_prefix"/share/locale");
  bind_textdomain_codeset("mms-pictures", nl_langinfo(CODESET));
#endif

  // header
  header = "Picture Options";
  translated_header = dgettext("mms-pictures", "Picture Options");
  save_name = "PictureOptions";

  // values

  PictureConfig *pic_conf = S_PictureConfig::get_instance();

#ifdef use_inotify
  if (pic_conf->p_picture_dirs_no_watch().size() > 0) {
#else
  if (pic_conf->p_picture_dirs_no_watch().size() > 0 || pic_conf->p_picture_dirs().size() > 0) {
#endif
    // reload
    vector<string> none;
    reload_p = new Option(false, dgettext("mms-pictures", "reload"), "reload", 0, none, none);
    val.push_back(reload_p);
  }

  // sorting order
  dir_order_p = go->dir_order();
  val.push_back(dir_order_p);

  if (conf->p_cdpath().size() > 1)
    val.push_back(go->cd_device);

  vector<string> slideshow_values;

  for (int i = 0; i < 120; ++i)
    slideshow_values.push_back(conv::itos(i));

  slideshow_p = new Option(false, dgettext("mms-pictures", "slideshow delay"), "slideshow delay", 10,
			   slideshow_values, slideshow_values);
  val.push_back(slideshow_p);

  vector<string> zoom_values;

  zoom_values.push_back(dgettext("mms-pictures", "no"));
  zoom_values.push_back(dgettext("mms-pictures", "yes"));

  vector<string> english_zoom_values;

  english_zoom_values.push_back("no");
  english_zoom_values.push_back("yes");

  zoom_p = new Option(false, dgettext("mms-pictures", "zoom to fullscreen"), "zoom to fullscreen", 0,
		      zoom_values, english_zoom_values);
  val.push_back(zoom_p);

  vector<string> preview_values;

  preview_values.push_back(dgettext("mms-pictures", "no"));
  preview_values.push_back(dgettext("mms-pictures", "yes"));

  vector<string> english_preview_values;

  english_preview_values.push_back("no");
  english_preview_values.push_back("yes");

  preview_p = new Option(false, dgettext("mms-pictures", "show previews"), "show previews", 1,
			 preview_values, english_preview_values);
  val.push_back(preview_p);

  vector<string> recurse_values;

  recurse_values.push_back(dgettext("mms-pictures", "no"));
  recurse_values.push_back(dgettext("mms-pictures", "yes"));

  vector<string> english_recurse_values;

  english_recurse_values.push_back("no");
  english_recurse_values.push_back("yes");

  recurse_p = new Option(false, dgettext("mms-pictures", "recurse dirs in fullscreen"), "recurse dirs in fullscreen", 0,
			 recurse_values, english_recurse_values);
  val.push_back(recurse_p);

  if (conf->p_idle_time() != 0) {
    vector<string> thumbnail_extract_values;

    thumbnail_extract_values.push_back(dgettext("mms-pictures", "no"));
    thumbnail_extract_values.push_back(dgettext("mms-pictures", "yes"));

    vector<string> english_thumbnail_extract_values;

    english_thumbnail_extract_values.push_back("no");
    english_thumbnail_extract_values.push_back("yes");

    thumbnail_extract_p = new Option(false, dgettext("mms-pictures", "bg thumbnail generation"), "bg thumbnail generation", 0,
				     thumbnail_extract_values, english_thumbnail_extract_values);
    val.push_back(thumbnail_extract_p);
  }

  vector<string> random_values;

  random_values.push_back(dgettext("mms-pictures", "no"));
  random_values.push_back(dgettext("mms-pictures", "yes"));

  vector<string> english_random_values;

  english_random_values.push_back("no");
  english_random_values.push_back("yes");

  random_p = new Option(false, dgettext("mms-pictures", "random order slideshow"), "random order slideshow", 0,
			 random_values, english_random_values);
  val.push_back(random_p);
}

Pictures::PicOpts::~PicOpts()
{
  save();

  delete dir_order_p;
  delete slideshow_p;
  delete zoom_p;
  delete preview_p;
  delete recurse_p;
  delete random_p;
  if (conf->p_idle_time() != 0) {
    delete thumbnail_extract_p;
  }
}

void Pictures::create_db()
{
  vector<string> fields;
  fields.push_back("version INTEGER NOT NULL");
  db.createTable("Schema", fields);

  db.execute("INSERT INTO Schema VALUES('3')");

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("parent INTEGER");
  fields.push_back("filename TEXT");
  fields.push_back("name TEXT");
  fields.push_back("lname TEXT");
  fields.push_back("is_folder INTEGER"); // 1 = true
  db.createTable("Folders", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("name TEXT");
  db.createTable("Group", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("pid INTEGER NOT NULL");
  fields.push_back("gid INTEGER NOT NULL");
  db.createTable("GPicture", fields);

  fields.clear();
  fields.push_back("id INTEGER PRIMARY KEY");
  fields.push_back("filename TEXT");
  fields.push_back("title TEXT");
  fields.push_back("comment TEXT");
  fields.push_back("date INTEGER");
  fields.push_back("gid INTEGERL");
  fields.push_back("rating INTEGER");
  fields.push_back("rotation INTEGER");
  db.createTable("Pictures", fields);

  create_indexes();
}

void Pictures::create_indexes()
{
  db.execute("CREATE INDEX idx_folders_name ON Folders (name)");
  db.execute("CREATE INDEX idx_folders_lname ON Folders (lname)");
  db.execute("CREATE INDEX idx_folders_parent ON Folders (parent)");
  db.execute("CREATE INDEX idx_folders_filename ON Folders (filename)");

  db.execute("CREATE INDEX idx_pictures_filename ON Pictures (filename)");
}

void Pictures::check_db()
{
  if (db.hasTable("Schema")) {
    // check for correct version
    SQLQuery *q = db.query("Schema", "SELECT version FROM %t");
    if (q && q->numberOfTuples() > 0) {
      SQLRow &row = (*q)[0];
      if (row["version"] != "3") {

	Print print(gettext("Upgrading pictures database schema"), Print::INFO);

	SQLQuery *dq = db.query("sqlite_master", "SELECT name FROM %t WHERE type='table'");
	if (dq) {
	  for (int i = 0; i < dq->numberOfTuples(); ++i) {
	    db.execute(("DROP TABLE " + (*dq)[i]["name"]).c_str());
	  }
	}

	delete dq;

	dq = db.query("sqlite_master", "SELECT name FROM %t WHERE type='index'");
	if (dq) {
	  for (int i = 0; i < dq->numberOfTuples(); ++i) {
	    db.execute(("DROP INDEX " + (*dq)[i]["name"]).c_str());
	  }
	}

	delete dq;

	create_db();
      }
    }
    delete q;
  } else {
    // create the whole db
    create_db();
  }
}

int Pictures::get_db_orientation(int db_id)
{
  int db_orientation = 0;

  db_mutex.enterMutex();

  SQLQuery *q = db.query("Folders", ("SELECT filename from %t WHERE id='" + conv::itos(db_id) + "'").c_str());

  if (q && q->numberOfTuples() > 0) {
    SQLQuery *q2 = db.query("Pictures", ("SELECT * FROM %t WHERE filename = '" + (*q)[0]["filename"] + "'").c_str());
    if (q2 && q2->numberOfTuples() > 0)
      db_orientation = conv::atoi((*q2)[0]["rotation"]);
    delete q2;
  }

  delete q;

  db_mutex.leaveMutex();

  return db_orientation;
}

void Pictures::set_db_orientation(int db_id)
{
  db_mutex.enterMutex();

  SQLQuery *q = db.query("Folders", ("SELECT filename from %t WHERE id='" + conv::itos(db_id) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    db.execute(("UPDATE Pictures SET rotation = '" + conv::itos(orientation) + "' WHERE filename = '" + (*q)[0]["filename"] + "'").c_str());
  }
  delete q;

  db_mutex.leaveMutex();
}

void Pictures::find_element_and_do_action(const Picture& picture)
{
  int p = 0;
  foreach (Picture& f, pic_list) {
    if (f == picture) {
      folders.top().second = p;
      if (search_mode)
	input_master->device->got_touch_action();
      else
	action();
      break;
    } else
      ++p;
  }
}

int Pictures::find_position_in_folder_list(const list<string>& files, const string& filename)
{
  int pos = 0;
  bool pos_found = false;
  foreach (const string& file, files) {
    vector<Picture> templist = rdir(file);
    pos = 0;
    foreach (Picture& picture, templist) {
      if (filename == picture.path) {
	pos_found = true;
	break;
      } else
	++pos;
    }
    if (pos_found)
      break;
  }

  if (!pos_found)
    return 0; // buggy
  else
    return pos;
}

bool Pictures::change_dir_to_id(int db_id)
{
  reset();

  db_mutex.enterMutex();

  SQLQuery *q = db.query("Folders", ("SELECT * FROM %t WHERE id='" + conv::itos(db_id) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {

    string filename = (*q)[0]["filename"];
    if (filename[filename.size() - 1] == '/')
      filename = filename.substr(0, filename.size()-1);
    string original_filename = filename;

    delete q;
    db_mutex.leaveMutex();

    string cur_dir_path;

    int pos;
    foreach (string& dir, picture_folders)
      if ((pos = filename.rfind(dir)) != string::npos) {
	cur_dir_path = filename.substr(0, dir.size()+1);
	filename = filename.substr(dir.size()+1);
      }

    if (!filename.empty()) {
      while ((pos = filename.find("/")) != string::npos) {
	folders.top().second = find_position_in_folder_list(folders.top().first, cur_dir_path + filename.substr(0, pos));

	string dir = cur_dir_path + filename.substr(0, pos+1);

	list<string> templs; templs.push_back(dir);
	assert(dir[dir.size()-1] == '/');
	folders.push(std::make_pair(templs, 0));
	cur_dir_path += filename.substr(0, pos+1);
	filename = filename.substr(pos+1);
      }
    }

    // we should now be in the correct folder, find the correct file
    folders.top().second = find_position_in_folder_list(folders.top().first, original_filename);

    load_current_dirs();

    return true;
  } else {
    delete q;
    db_mutex.leaveMutex();
    return false;
  }
}

void Pictures::reenter(int db_id)
{
  if (change_dir_to_id(db_id))
    mainloop();
}

vector<std::pair<string, int> > Pictures::gen_search_list(const string& search_word)
{
  string parent = "";

  if (input_master->current_saved_map() == "pictures" && search_depth == dgettext("mms-pictures", "current folder")) {
    string parent_sql_str = "SELECT DISTINCT id FROM %t WHERE";

    list<string> current_folders = folders.top().first;
    bool first = true;
    foreach (string& folder, current_folders)
      if (first) {
	parent_sql_str += " filename='" + folder + "'";
	first = false;
      } else
	parent_sql_str += " OR filename='" + folder + "'";

    string parent_ids = "";

    db_mutex.enterMutex();

    SQLQuery *q0 = db.query("Folders", parent_sql_str.c_str());

    if (q0) {
      for (int i = 0; i < q0->numberOfTuples(); ++i) {
	SQLRow &row = (*q0)[i];
	if (i > 0)
	  parent_ids += ", " + row["id"];
	else
	  parent_ids += row["id"];
      }
    }
    delete q0;

    db_mutex.leaveMutex();

    parent = "parent in (" + parent_ids + ") AND";
  }

  std::vector<std::pair<string, int> > result;

  db_mutex.enterMutex();

  SQLQuery *q = db.query("Folders", ("SELECT DISTINCT id, parent, filename, name, is_folder FROM %t WHERE " + parent + " fuzzycmp('" + search_word + "', lname, 1)").c_str());

  if (q) {

    for (int i = 0; i < q->numberOfTuples(); ++i) {

      SQLRow &row = (*q)[i];

      bool result_ok = false;

      if (search_depth == dgettext("mms-pictures", "current folder")) {
	foreach (string& folder, folders.top().first)
	  if (row["filename"].find(folder) != string::npos) {
	    result_ok = true;
	    break;
	  }
      } else
	result_ok = true;

      if (result_ok && row["parent"] != "0") {
	string out = row["name"];
	result.push_back(std::make_pair(string_format::convert(out), conv::atoi(row["id"])));
      }
    }
  }
  delete q;

  db_mutex.leaveMutex();

  return result;
}

bool Pictures::file_sort::operator()(const Simplefile& lhs, const Simplefile& rhs)
{
  return sort_order(lhs.lowercase_name, rhs.lowercase_name, lhs.type == "dir", rhs.type == "dir",
		    get_class<Pictures>(dgettext("mms-pictures", "Pictures"))->get_opts()->dir_order() == gettext("directories first"));
}

void Pictures::action_no_find_recursion()
{
  if (vector_lookup(pic_list, folders.top().second).type != "dir")
    pictures_fullscreen();
  else
    enter_dir();
}

// commands
void Pictures::action()
{
  if (vector_lookup(pic_list, folders.top().second).type != "dir" &&
      conv::stob(opts.recurse_in_fullscreen())) {

    recurse_files = parse_dirs_recursion();

    if (find_recursion_pos())
      pictures_fullscreen();

  } else
    action_no_find_recursion();
}

bool Pictures::enter_dir()
{
  if(vector_lookup(pic_list, folders.top().second).type != "dir") {
    input_master->set_map("pictures_fullscreen");
    pictures_fullscreen();
    input_master->set_map("pictures");
  } else {

    // FIXME: remove

    // go into the directory.
    string temp_cur_dir = vector_lookup(pic_list, folders.top().second).path;

    // get the total number of files:
    int nr_files = 0;
    for (file_iterator<file_t, default_order> i (temp_cur_dir); i != i.end (); i.advance(false))
      if (!isDirectory(i->getName()))
	++nr_files;

    ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

    // disable screensaver if it's running
    bool audio_fullscreen_running = true;

    if (!screen_updater->timer.status("audio_fullscreen"))
      audio_fullscreen_running = false;
    else
      screen_updater->timer.deactivate("audio_fullscreen");

    screen_updater->status_progressbar = 0;
    screen_updater->total_progressbar = nr_files;
    screen_updater->header = dgettext("mms-pictures", "Generating thumbnails");

    screen_updater->timer.add(TimeElement("progressbar", &check_progressbar, &print_progressbar));

    if (conf->p_priority_change())
      nice(19); //Lower priority to keep audio from skipping

    // process picture dir
    vector<Picture> templist = rdir(temp_cur_dir);

    if (conf->p_priority_change())
      nice(-19);

    screen_updater->timer.del("progressbar");
    cleanup_progressbar();

    // reenable screensaver
    if (audio_fullscreen_running)
      screen_updater->timer.activate("audio_fullscreen");

    if (templist.size() != 0) {
      list<string> templs;
      if (temp_cur_dir[temp_cur_dir.size()-1] != '/')
	templs.push_back(temp_cur_dir + '/');
      else
	templs.push_back(temp_cur_dir);
      folders.push(std::make_pair(templs, 0));
      pic_list = templist;

      return true;
    } else {
      DialogWaitPrint pdialog(dgettext("mms-pictures", "Folder is empty"), 1000);
      return false;
    }

    S_BackgroundUpdater::get_instance()->run_once(boost::bind(&Pictures::check_for_changes, this));
  }
  return true;
}

void Pictures::go_back()
{
  if (folders.size() > 1) {
    folders.pop();

    load_current_dirs();

  } else
    exit();
}

void Pictures::load_current_dirs()
{
  pic_list = parse_dir(folders.top().first);

  if (folders.top().first.size() > 1 && folders.size() == 1)
    std::sort(pic_list.begin(), pic_list.end(), file_sort());
}

bool Pictures::find_recursion_pos()
{
  bool found_match = false;
  list<string> dirs = folders.top().first;
  pos_recursion = 0;

  // find the first image compared to the current position
  string cur_file = vector_lookup(pic_list, folders.top().second).path;
  foreach (Picture& picture, recurse_files)
    if (picture.path.substr(0, cur_file.size()) == cur_file) {
      found_match = true;
      break;
    } else
      ++pos_recursion;

  return found_match;
}

void Pictures::recurse_dirs()
{
  in_fullscreen = true;

  // create list, find the next image compared to where we stand currently
  recurse_files = parse_dirs_recursion();

  bool found_match = find_recursion_pos();

  if (found_match) {

    if (recurse_files.size() > 1) {
      if (pos_recursion == 0)
	pos_recursion = recurse_files.size() -1;
      else
	pos_recursion -= 1;
    }

    next_no_skip_folders();

    pictures_fullscreen();
  }
}

bool Pictures::reload_dir(const string& path)
{
  vector<Picture> hdfiles = rdir_hd(path, false);

  vector<Picture> dbfiles = rdir(path);

  // find what is new and what is removed
  vector<string> new_list;
  foreach (Picture& picture, hdfiles)
    new_list.push_back(picture.path);

  vector<string> old_list;
  foreach (Picture& picture, dbfiles)
    old_list.push_back(picture.path);

  vector<string> new_files;
  vector<string> removed_files;

  folder_difference(new_list, old_list, new_files, removed_files);

  // insert new into db
  foreach (string& path, new_files) {
    string original_path = path;
    if (path[path.size() - 1] == '/')
      path = path.substr(0, path.size()-1);

    string::size_type pos_backslash = path.rfind('/');
    if (pos_backslash != string::npos) {
      path = path.substr(0, pos_backslash+1);
    }

    insert_file_into_db(original_path, path);
  }

  // remove old from db
  foreach (string& path, removed_files) {
    db_mutex.enterMutex();

    char *tmp = sqlite3_mprintf("DELETE FROM Folders WHERE filename LIKE '%q%%'", path.c_str());
    db.execute(tmp);
    sqlite3_free(tmp);

    tmp = sqlite3_mprintf("DELETE FROM Pictures WHERE filename LIKE '%q%%'", path.c_str());
    db.execute(tmp);
    sqlite3_free(tmp);

    db_mutex.leaveMutex();
  }

  return (removed_files.size() > 0 || new_files.size() > 0);
}

void Pictures::check_for_changes()
{
  bool reload = false;

  list<string> top_folders = folders.top().first;

  foreach (string& dir, top_folders)
    if (reload_dir(dir))
      reload = true; // no break since we want to update the db for all folders

  reload_dirs = reload;
}

void Pictures::reload_current_dirs()
{
  load_current_dirs();

  if (pic_list.size() == 0)
    go_back();
}

void Pictures::update_db_current_dirs()
{
  foreach (string& dir, folders.top().first)
    reload_dir(dir);
}

void Pictures::options()
{
  visible = false;

  S_BusyIndicator::get_instance()->idle();

  if (opts.mainloop()) {

    update_db_current_dirs();

    reload_current_dirs();

    if (folders.top().second > pic_list.size()-1)
      folders.top().second = 0;

    thumbnailed_all_files = false;
  }

  opts.save();

  visible = true;
}

string Pictures::id_to_filename(int db_id)
{
  string path;

  db_mutex.enterMutex();

  SQLQuery *q = db.query("Folders", ("SELECT filename FROM %t WHERE id='" + conv::itos(db_id) + "'").c_str());
  if (q && q->numberOfTuples() > 0)
    path = (*q)[0]["filename"];
  else
    assert(false);

  db_mutex.leaveMutex();

  return path;
}

bool Pictures::search_is_graphical_view()
{
  return true;
}

void Pictures::search_func()
{
  S_BusyIndicator::get_instance()->idle();

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  search_mode = true;
  search_help_offset = -1;

  print_marked_search_letter();

  list<string> empty;
  empty.push_back("");

  screen_updater->trigger.add(TriggerElement("search marker", empty, boost::bind(&Pictures::print_marked_search_letter, this),
					     boost::bind(&Pictures::clean_up_search_print, this)));
  screen_updater->timer.add(TimeElement("search marker", boost::bind(&Pictures::check_search_letter, this),
					boost::bind(&Pictures::print_marked_search_letter, this)));

  int pos = input_master->device->graphical_search<Picture>(pic_list,
							    boost::bind(&Pictures::print, this, _1),
							    boost::bind(&Pictures::gen_search_list, this, _1),
							    boost::bind(&Pictures::id_to_filename, this, _1),
							    boost::bind(&Pictures::search_is_graphical_view, this),
							    images_per_row, search_mode, search_str, lowercase_search_str,
							    search_depth, offset, search_help_offset,
							    S_BusyIndicator::get_instance());

  screen_updater->trigger.del("search marker");
  screen_updater->timer.del("search marker");

  if (pos != -1 && search_depth == dgettext("mms-pictures", "current folder"))
    folders.top().second = pos;
  else if (pos != -1)
    change_dir_to_id(pos);
}

void Pictures::exit()
{
  exit_loop = true;
}

void Pictures::exit_fullscreen()
{
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  screen_updater->timer.del("pictures");

  exit();

  in_fullscreen = false;

  if (conv::stob(opts.recurse_in_fullscreen())) {
    if (folders.size() > 1) {
      folders.pop();
      load_current_dirs();
      enter_dir();
    }

    string recursion_file = vector_lookup(recurse_files, pos_recursion).path;

    int pos = 0;
    foreach (Picture& picture, pic_list) {
      if (!isDirectory(picture.path)) {
	if (picture.path == recursion_file) {
	  folders.top().second = pos;
	  return;
	}
      }
      ++pos;
    }

    // file not found
    folders.top().second = 0;
  }
}

void Pictures::find_recursion_file()
{
  string next_file = vector_lookup(recurse_files, pos_recursion).path;
  string next_path = next_file;
  string::size_type pos;
  if ((pos = next_path.rfind('/')) != string::npos)
    next_path = next_path.substr(0, pos);

  vector<string> tested_dirs;

  while (true) {

    // quick check

    // next
    if (folders.top().second + 1 <= pic_list.size() - 1) {
      if (vector_lookup(pic_list, folders.top().second + 1).path == next_file) {
	folders.top().second += 1;
	return;
      }
    }

    // prev
    if (folders.top().second - 1 >= 0 && pic_list.size() >= folders.top().second) {
      if (vector_lookup(pic_list, folders.top().second - 1).path == next_file) {
	folders.top().second -= 1;
	return;
      }
    }

    // check if on current level
    list<string> dirs = folders.top().first;

    int file_pos = 0;
    foreach (Picture& picture, pic_list) {
      if (!isDirectory(picture.path)) {
	if (picture.path == next_file) {
	  folders.top().second = file_pos;
	  return;
	}
      }
      ++file_pos;
    }

    // try to find the correct subdir in current dir
    bool found_dir = false;
    file_pos = 0;
    foreach (Picture& picture, pic_list) {
      string cur_file = picture.path;
      if (cur_file.size() <= next_path.size()) {
	if (next_path.substr(0, cur_file.size()) == cur_file &&
	    !already_tested(tested_dirs, cur_file)) {
	  if (isDirectory(cur_file)) {
	    tested_dirs.push_back(cur_file);
	    folders.top().second = file_pos;
	    enter_dir();
	    found_dir = true;
	    break;
	  }
	}
      }
      ++file_pos;
    }

    if (!found_dir) {
      // it must be on a level below
      go_back();
    }
  }
}

bool Pictures::already_tested(const vector<string>& tested_dirs, string dir)
{
  foreach (const string& tested_dir, tested_dirs)
    if (tested_dir == dir)
      return true;
  return false;
}

// helper functions, to minimize code copying, functions used in mainloops.
void Pictures::prev()
{
  if (folders.top().second == 0)
    folders.top().second = pic_list.size()-1;
  else
    --(folders.top().second);
}

void Pictures::next()
{
  folders.top().second = (folders.top().second+1)%pic_list.size();
}

void Pictures::prev_no_skip_folders()
{
  if (pos_recursion != 0)
    --pos_recursion;
  else
    pos_recursion = recurse_files.size() - 1;

  find_recursion_file();
}

void Pictures::prev_skip_folders()
{
  do {
    prev();
  } while (vector_lookup(pic_list, folders.top().second).type == "dir");
}

void Pictures::next_no_skip_folders()
{
  if (pos_recursion != recurse_files.size() - 1)
    ++pos_recursion;
  else
    pos_recursion = 0;

  find_recursion_file();
}

void Pictures::next_skip_folders()
{
  do {
    next();
  } while (vector_lookup(pic_list, folders.top().second).type == "dir");
}

string Pictures::mainloop()
{
  visible = true;

  input_master->set_map("pictures");

  search_top_size = 30;

  Input input;

  bool update_needed = true;

  BusyIndicator *busy_indicator = S_BusyIndicator::get_instance();
  Shutdown *sd = S_Shutdown::get_instance();

  Render *render = S_Render::get_instance();
  int curlayer = render->device->get_current_layer();
  render->device->switch_to_layer(1);
  render->device->animation_section_begin();
  render->device->animation_zoom(0,0,1,1,20,1);
  render->device->animation_fade(0,1,40,1);
  render->device->animation_fade(1,0,40,curlayer);
  render->device->animation_section_end();

  while (!exit_loop)
    {
      if (reload_dirs) {
	reload_current_dirs();
	update_needed = true;
	reload_dirs = false;
      }

      if (update_needed)
	print(pic_list);

      input = input_master->device->get_input_busy_wrapped(busy_indicator);

      if (sd->is_enabled()) {
      	sd->cancel();
      	continue;
      }

      update_needed = true;

      if (input.key == "touch_input") {
	S_Touch::get_instance()->run_callback();
	continue;
      }

      if (input.command == "prev")
 	{
	  if (pic_list.size() > images_per_row) {
	    for (int i = 0; i < images_per_row; ++i) {
	      if (folders.top().second == 0) {
		folders.top().second = pic_list.size()-1;
		i += images_per_row-pic_list.size()%images_per_row;
	      }
	      else
		--(folders.top().second);
	    }
	  }
 	}
      else if (input.command == "next")
 	{
	  if (pic_list.size() > images_per_row) {
	    for (int i = 0; i < images_per_row; ++i) {
	      if (folders.top().second == pic_list.size()-1 && i == 0) {
		folders.top().second = 0;
		break;
	      } else if (folders.top().second == pic_list.size()-1) {
		break;
	      } else {
		++(folders.top().second);
	      }
	    }
	  }
 	}
      else if (input.command == "left")
 	{
	  prev();
 	}
      else if (input.command == "right")
 	{
	  next();
 	}
      else if (input.command == "page_up")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  if (pic_list.size() > jump) {
	    int diff = folders.top().second-jump;
	    if (folders.top().second == 0)
	      folders.top().second = (pic_list.size()-1)+diff;
	    else if (diff < 0)
	      folders.top().second = 0;
	    else
	      folders.top().second = diff;
	  }
 	}
      else if (input.command == "page_down")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  if (pic_list.size() > jump) {
	    if (folders.top().second >= (pic_list.size() - jump) && folders.top().second != (pic_list.size()-1))
	      folders.top().second = pic_list.size()-1;
	    else
	      folders.top().second = (folders.top().second+jump)%pic_list.size();
	  }
 	}
      else if (input.command == "recurse_dirs" && conv::stob(opts.recurse_in_fullscreen()))
	{
	  recurse_dirs();
	}
      else if (input.command == "action")
	{
	  action();
	}
      else if (input.command == "back")
	{
	  go_back();
	}
      else if (input.command == "second_action")
	{
	  ExtraMenu em;

	  string t = dgettext("mms-pictures", "Show picture in fullscreen");

	  if (vector_lookup(pic_list, folders.top().second).type == "dir")
	    t = dgettext("mms-pictures", "Enter directory");

	  em.add_item(ExtraMenuItem(t, input_master->find_shortcut("action"),
				    boost::bind(&Pictures::action, this)));

	  if (conv::stob(opts.recurse_in_fullscreen())) {
	    if (vector_lookup(pic_list, folders.top().second).type == "dir")
	      em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Recurse all folders on current level"),
					input_master->find_shortcut("recurse_dirs"),
					boost::bind(&Pictures::recurse_dirs, this)));
	  }

	  em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Search"), input_master->find_shortcut("search"),
				    boost::bind(&Pictures::search_func, this)));
	  em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Options"), input_master->find_shortcut("options"),
				    boost::bind(&Pictures::options, this)));

	  if (folders.size() > 1) {
	    em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Go up one directory"), input_master->find_shortcut("back"),
				      boost::bind(&Pictures::go_back, this)));
	    em.add_item(ExtraMenuItem(dgettext("mms-game", "Return to startmenu"), input_master->find_shortcut("startmenu"),
				      boost::bind(&Pictures::exit, this)));
	  } else
	    em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Return to startmenu"), input_master->find_shortcut("back"),
				      boost::bind(&Pictures::exit, this)));

	  foreach (ExtraMenuItem& item, global->menu_items)
	    em.add_persistent_item(item);

	  em.mainloop();
	}
      else if (input.command == "startmenu")
 	{
	  exit();
	}
      else if (input.command == "options")
	{
	  options();
	}
      else if (input.command == "search")
	{
	  search_func();
	}
      else
	update_needed = !global->check_commands(input);
    }

  exit_loop = false;

  visible = false;

  render->device->layer_active(false);
  render->device->switch_to_layer(curlayer);
  render->device->animation_section_begin();
  render->device->reset_layout_attribs_nowait();
  render->device->animation_fade(0,1,80,curlayer);
  render->device->animation_section_end();

  return "";
}

vector<string> Pictures::images_in_dir(const list<string>& dirs)
{
  std::stack<string> queued_dirs;
  vector<string> image_files;

  libfs_set_folders_first(opts.dir_order());

  foreach (const string& dir, dirs) {
    queued_dirs.push(dir);
    while (!queued_dirs.empty()) {
      string cur_dir = queued_dirs.top();
      queued_dirs.pop();
      for (file_iterator<file_t, default_order> i (cur_dir); i != i.end (); i.advance(false)) {

	string cur_file = filesystem::FExpand(i->getName());

	// exclude folders ?
	if (pic_conf->p_excluded_picture_dirs().size() > 0) {
	  list<string> excluded = pic_conf->p_excluded_picture_dirs();
	  bool stop = false;

	  for (list<string>::const_iterator epd = excluded.begin(), epd_end = excluded.end();
	       epd != epd_end; ++epd)
	    if (cur_file.substr(cur_file.rfind("/")+1) == *epd) {
	      stop = true;
	      break;
	    }

	  if (stop)
	    continue;
	}

	if (isDirectory(cur_file))
	  queued_dirs.push(cur_file);
	else
	  image_files.push_back(cur_file);
      }
    }
  }

  return image_files;
}


void Pictures::background_thumbnails()
{
  if (!thumbnailed_all_files && (conf->p_idle_time() > 0) &&
      (opts.thumbnail_extract() == dgettext("mms-pictures", "yes"))) {

    int nr_extracts = 3;

    // check if we are idle
    int idle_time = time(0)-conf->p_last_key();
    if (idle_time < 3)
      return;
    else if (idle_time > 60 * conf->p_idle_time())
      nr_extracts = 4;

    if (all_image_files.empty()) {
      all_image_files = images_in_dir(picture_folders);
      // optimization, we don't want to both make the list and start extracting
      return;
    }

    if (conf->p_priority_change())
      nice(19); //Lower priority to keep audio from skipping

    while (!all_image_files.empty()) {

      bool extracted = false;

      if (!file_exists(render->image_get_fullpath(all_image_files.back(), image_width, image_height))) {
	extracted = true;
	render->image_mut.enterMutex();
	render->device->wait_for_not_drawing();
	render->create_scaled_image_wrapper(all_image_files.back(), image_width, image_height);
	render->image_mut.leaveMutex();
      }

      if (!file_exists(render->image_get_fullpath(all_image_files.back(), conf->p_h_res(), conf->p_v_res()))) {
	extracted = true;
	render->image_mut.enterMutex();
	render->device->wait_for_not_drawing();
	render->create_scaled_image_wrapper(all_image_files.back(), conf->p_h_res(), conf->p_v_res());
	render->image_mut.leaveMutex();
      }

      if (nr_extracts == 0) {
	if (conf->p_priority_change())
	  nice(-19);
	return;
      } else if (extracted)
	--nr_extracts;

      all_image_files.erase(all_image_files.end());
    }

    if (conf->p_priority_change())
      nice(-19);

    thumbnailed_all_files = true;
  }
}

int Pictures::check_thumbnail()
{
  return 3000; // ms
}

string Pictures::get_name_from_file(const Simplefile& s)
{
  return s.name;
}

bool Pictures::search_compare(const Simplefile& s)
{
  return (s.lowercase_name.substr(0, search_str.size()) == lowercase_search_str);
}

void Pictures::prepare_and_show_fullscreen()
{
  string left_path, right_path;

  if (conv::stob(opts.recurse_in_fullscreen())) {

    int left_pos_recursion;
    if (pos_recursion != 0)
      left_pos_recursion = pos_recursion - 1;
    else
      left_pos_recursion = recurse_files.size() - 1;

    left_path = vector_lookup(recurse_files, left_pos_recursion).path;
    right_path = vector_lookup(recurse_files, (pos_recursion + 1) % recurse_files.size()).path;

  } else {

    int left = folders.top().second;
    do {
      left -= 1;
      if (left == -1)
	left=pic_list.size()-1;
    } while (vector_lookup(pic_list, left).type == "dir");

    int right = folders.top().second;
    do {
      right += 1;
      if (right == pic_list.size())
	right=0;
    } while (vector_lookup(pic_list, right).type == "dir");

    left_path = pic_list[left].path;
    right_path = pic_list[right].path;
  }

  if (conf->p_priority_change())
    nice(19); //Lower priority to keep audio from skipping

  print_fullscreen(pic_list[folders.top().second], left_path, right_path);

  if (conf->p_priority_change())
    nice(-19);
}

void Pictures::zoom()
{
  if (zoom_level == 1) {

    // calculate zoom position
    render->image_mut.enterMutex();
    PFSObj p = PFSObj(vector_lookup(pic_list, folders.top().second).path,
		      false, orientation, zoom_level, zoom_position, 1, 1, 1);
    render->image_mut.leaveMutex();

    // don't zoom small images
    if (p.real_w < conf->p_h_res() && p.real_h < conf->p_v_res())
      return;

    zoom_level = 2;

    int pic_res_w = p.real_w*zoom_level;
    max_pos_v = 3; // default
    if (pic_res_w <= conf->p_h_res())
      max_pos_v = 1;
    else if (pic_res_w <= int(1.5*conf->p_h_res()))
      max_pos_v = 2;

    int pic_res_h = p.real_h*zoom_level;
    max_pos_h = 3; // default
    if (pic_res_h <= conf->p_v_res())
      max_pos_h = 1;
    else if (pic_res_h <= int(1.5*conf->p_v_res()))
      max_pos_h = 2;

    zoom_pos_h = int(max_pos_h/2)+1;
    zoom_pos_v = int(max_pos_v/2)+1;

    zoom_position = (zoom_pos_h-1)*max_pos_v+zoom_pos_v-1;

  } else {
    zoom_level = 1;
    zoom_position = 1;
  }
}

void Pictures::suspend()
{
  pause = !pause;
}

void Pictures::rotate_right()
{
  orientation = (orientation+1)%4;
  set_db_orientation_lookup();
}

void Pictures::rotate_left()
{
  orientation = (orientation == 0) ? 3 : orientation - 1;
  set_db_orientation_lookup();
}

void Pictures::options_fullscreen()
{
  bool recursion_before = conv::stob(opts.recurse_in_fullscreen());

  options();

  if (!recursion_before && conv::stob(opts.recurse_in_fullscreen())) {
    recurse_files = parse_dirs_recursion();
    find_recursion_pos();
  }
}

void Pictures::deactivate_updaters()
{
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  screen_updater->timer.deactivate("pictures");
}

void Pictures::activate_updaters()
{
  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  if (opts.slideshow() != "0")
    screen_updater->timer.activate("pictures");
}

int Pictures::get_db_orientation_lookup()
{
  if (conv::stob(opts.recurse_in_fullscreen()))
    return get_db_orientation(vector_lookup(recurse_files, pos_recursion).db_id);
  else
    return get_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
}

void Pictures::set_db_orientation_lookup()
{
  if (conv::stob(opts.recurse_in_fullscreen()))
    return set_db_orientation(vector_lookup(recurse_files, pos_recursion).db_id);
  else
    return set_db_orientation(vector_lookup(pic_list, folders.top().second).db_id);
}

void Pictures::fullscreen_left()
{
  if (conv::stob(opts.recurse_in_fullscreen()))
    prev_no_skip_folders();
  else
    prev_skip_folders();

  orientation = get_db_orientation_lookup();
}

void Pictures::fullscreen_right()
{
  if (conv::stob(opts.recurse_in_fullscreen()))
    next_no_skip_folders();
  else
    next_skip_folders();

  orientation = get_db_orientation_lookup();
}

void Pictures::new_random_fullscreen(const int list_size)
{
  if (list_size > 1) {
    Rand *rand = S_Rand::get_instance();

    int r = rand->number(list_size);

    if (conv::stob(opts.recurse_in_fullscreen())) {
      while (pos_recursion == r)
	r = rand->number(list_size);

      pos_recursion = r;
      find_recursion_file();
    } else {
      while (folders.top().second == r)
	r = rand->number(list_size);

      folders.top().second = r;
    }

    orientation = get_db_orientation_lookup();
  }
}

void Pictures::pictures_fullscreen()
{
  in_fullscreen = true;

  Input input;

  orientation = get_db_orientation_lookup();

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();

  screen_updater->timer.add(TimeElement("pictures", boost::bind(&Pictures::check_picture_time, this),
					boost::bind(&Pictures::pictures_check, this)));

  if (opts.slideshow() == "0")
    screen_updater->timer.deactivate("pictures");

  // disable screensaver if it's running
  bool audio_fullscreen_running = true;

  if (!screen_updater->timer.status("audio_fullscreen"))
    audio_fullscreen_running = false;
  else
    screen_updater->timer.deactivate("audio_fullscreen");

  bool update_needed = true;

  BusyIndicator *busy_indicator = S_BusyIndicator::get_instance();

#ifdef use_notify_area
  nArea->disable();
#endif

  while (!exit_loop)
    {
      global->set_playback_offset(image_width + 10);

      prepare_and_show_fullscreen();

      input = input_master->device->get_input_busy_wrapped(busy_indicator);

      last_press = time(0);

      update_needed = true;

      global->set_playback_offset(0);

      if (input.key == "touch_input") {
	S_Touch::get_instance()->run_callback();
	continue;
      }

      if (zoom_level != 1) {

	if (input.command == "prev") {
	  if (zoom_pos_h != 1)
	    --zoom_pos_h;
	} else if (input.command == "next") {
	  if (zoom_pos_h < max_pos_h)
	    ++zoom_pos_h;
	} else if (input.command == "left") {
	  if (zoom_pos_v != 1)
	    --zoom_pos_v;
	} else if (input.command == "right") {
	  if (zoom_pos_v < max_pos_v)
	    ++zoom_pos_v;
	} else if (input.command == "action" || (input.command == "back" && input.mode == "general")) {
	  zoom_level = 1;
	  zoom_position = 1;
	  activate_updaters();
	  continue;
	}

	// calculate zoom_position
	zoom_position = (zoom_pos_h-1)*max_pos_v + zoom_pos_v - 1;

	continue;
      }

      if (input.command == "back" && input.mode == "general")
	{
	  exit_fullscreen();
	}
      else if (input.command == "left")
	{
	  // we don't want to change left or right since they are ok for touch
	  // screen even in random mode
	  if (conv::stob(opts.slideshow_random())) {
	    int list_size = pic_list.size();
	    if (conv::stob(opts.recurse_in_fullscreen()))
	      list_size = recurse_files.size();
	    new_random_fullscreen(list_size);
	  } else
	    fullscreen_left();
	}
      else if (input.command == "right")
	{
	  if (conv::stob(opts.slideshow_random())) {
	    int list_size = pic_list.size();
	    if (conv::stob(opts.recurse_in_fullscreen()))
	      list_size = recurse_files.size();
	    new_random_fullscreen(list_size);
	  } else
	    fullscreen_right();
	}
      else if (input.command == "page_up")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  int list_size = pic_list.size();
	  if (conv::stob(opts.recurse_in_fullscreen()))
	    list_size = recurse_files.size();

	  if (conv::stob(opts.slideshow_random())) {
	    new_random_fullscreen(list_size);
	  } else {
	    if (list_size > jump) {
	      for (int i = 0; i < jump; ++i) {
		if (conv::stob(opts.recurse_in_fullscreen()))
		  prev_no_skip_folders();
		else
		  prev_skip_folders();
	      }

	      orientation = get_db_orientation_lookup();
	    }
	  }
 	}
      else if (input.command == "page_down")
 	{
	  int jump = images_per_row*rows;

	  if (search_mode)
	    jump = images_per_row*rows_search;

	  int list_size = pic_list.size();
	  if (conv::stob(opts.recurse_in_fullscreen()))
	    list_size = recurse_files.size();

	  if (conv::stob(opts.slideshow_random())) {
	    new_random_fullscreen(list_size);
	  } else {
	    if (list_size > jump) {
	      for (int i = 0; i < jump; ++i) {
		if (conv::stob(opts.recurse_in_fullscreen()))
		  next_no_skip_folders();
		else
		  next_skip_folders();
	      }

	      orientation = get_db_orientation_lookup();
	    }
	  }
 	}
      else if (input.command == "next")
	{
	  rotate_right();
	}
      else if (input.command == "prev")
	{
	  rotate_left();
	}
      else if (input.command == "action")
	{
	  zoom();

	  if (zoom_level != 1)
	    deactivate_updaters();
	  else
	    activate_updaters();
	}
      else if (input.command == "suspend")
	{
	  deactivate_updaters();
	  suspend();
	  activate_updaters();
	}
      else if (input.command == "second_action")
	{
	  ExtraMenu em;

	  em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Zoom"), input_master->find_shortcut("action"),
				    boost::bind(&Pictures::zoom, this), true));

	  string slideshow_status = dgettext("mms-pictures", "Suspend slideshow");

	  if (pause)
	    slideshow_status = dgettext("mms-pictures", "Resume slideshow");

	  em.add_item(ExtraMenuItem(slideshow_status, input_master->find_shortcut("suspend"),
				    boost::bind(&Pictures::suspend, this)));

	  em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Rotate left"), input_master->find_shortcut("prev"),
				    boost::bind(&Pictures::rotate_left, this)));

	  em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Rotate right"), input_master->find_shortcut("next"),
				    boost::bind(&Pictures::rotate_right, this)));

	  em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Options"), input_master->find_shortcut("options"),
				    boost::bind(&Pictures::options_fullscreen, this)));

	  em.add_item(ExtraMenuItem(dgettext("mms-pictures", "Leave fullscreen"),
				    input_master->find_shortcut("back"),
				    boost::bind(&Pictures::exit_fullscreen, this)));

	  foreach (ExtraMenuItem& item, global->menu_items)
	    em.add_persistent_item(item);

	  deactivate_updaters();

	  global->set_playback_offset(image_width + 10);

	  em.mainloop();

	  if (zoom_level == 1)
	    activate_updaters();
	}
      else if (input.command == "options")
	{
	  deactivate_updaters();
	  options_fullscreen();
	  activate_updaters();
	}
      else
	update_needed = !global->check_commands(input);
    }

  if (zoom_level != 1) {
    zoom_level = 1;
    zoom_position = 1;
  }

  exit_loop = false;

  global->set_playback_offset(0);
  S_SSaverobj::get_instance()->ResetCounter();

  if (audio_fullscreen_running)
    screen_updater->timer.activate("audio_fullscreen");

#ifdef use_notify_area
  nArea->enable();
#endif

}

void Pictures::print(const vector<Picture>& cur_list)
{
  render->prepare_new_image();

  render->current.add(new PObj(themes->pictures_background, 0, 0, 0, SCALE_FULL));

  PObj *back = new PObj(themes->startmenu_pictures_dir, 25, 10, 2, NOSCALING);

  render->current.add(back);

  S_Touch::get_instance()->register_area(TouchArea(rect(back->x, back->y, back->w, back->h), back->layer, boost::bind(&Pictures::go_back, this)));

  if (themes->show_header) {
    string header = dgettext("mms-pictures", "Pictures");

    if (folders.size() > 1) {
      string top_folder = folders.top().first.front();

      if (top_folder[top_folder.size()-1] == '/')
	top_folder = top_folder.substr(0, top_folder.size()-1);

      assert(top_folder.rfind('/') != string::npos);
      header += " - " + top_folder.substr(top_folder.rfind('/')+1);
      string_format::format_to_size(header, "Vera/28", conf->p_h_res()-220, false);
    }

    TObj *t = new TObj(header, "Vera/28", 100, 5,
		       themes->pictures_header_font1, themes->pictures_header_font2,
		       themes->pictures_header_font3, 2);

    render->current.add(t);

    S_Touch::get_instance()->register_area(TouchArea(rect(t->x, t->y, t->w, t->h), t->layer, global->toggle_playlist));
  }

  if (search_mode) {

    int y = 60;
    if (conf->p_h_res() == 720) {
      y -= 3;
    }

    if (offset == -1)
      render->current.add(new RObj(0, y, conf->p_h_res(), 45, 0, 0, 0, 215, 2));
    else
      render->current.add(new RObj(0, y, conf->p_h_res(), 45,
				   themes->general_search_rectangle_color1,
				   themes->general_search_rectangle_color2,
				   themes->general_search_rectangle_color3,
				   themes->general_search_rectangle_color4, 2));

    render->current.add(new PObj(themes->general_search, 47, y, 3, NOSCALING));

    int max_x = conf->p_h_res()-250;

    string sw = search_str;

    string s = dgettext("mms-pictures", "Search: ");
    int x_size = string_format::calculate_string_width(s, "Vera/22");
    string_format::format_to_size(sw, "Vera/22", max_x-x_size, true);

    render->current.add(new TObj(s + sw, "Vera/22", 100, y+5,
				 themes->search_font1, themes->search_font2,
				 themes->search_font3, 3));

    render->current.add(new TObj(search_depth, "Vera/17", 50+max_x+25, y+10, themes->search_font1,
				 themes->search_font2, themes->search_font3, 3));
  }

  // print the range
  int pos, start, pos_in_list, cur_rows, cur_image_height_all;

  if (search_mode) {
    if (offset == -1)
      pos_in_list = 0;
    else
      pos_in_list = offset;
    cur_rows = rows_search;
    cur_image_height_all = image_height_all_search;
  } else {
    pos_in_list = folders.top().second;
    cur_rows = rows;
    cur_image_height_all = image_height_all;
  }

  int nr_in_last_row = cur_list.size()%images_per_row;
  if (nr_in_last_row == 0)
    nr_in_last_row = images_per_row;

  if (cur_list.size() > cur_rows*images_per_row) {
    if (pos_in_list < std::ceil(double(cur_rows)/2.0)*images_per_row) { // first two rows
      pos = pos_in_list;
      start = 0;
    } else if (pos_in_list >= cur_list.size() - (images_per_row+nr_in_last_row)) { // last two rows
      start = cur_list.size()-((cur_rows-1)*images_per_row+nr_in_last_row);
      pos = pos_in_list-start;
    } else {
      pos = pos_in_list%images_per_row+images_per_row;
      if (pos == images_per_row-1)
	pos = 2*images_per_row-1;
      start = pos_in_list-pos;
    }
  } else {
    pos = pos_in_list;
    start = 0;
  }

  int x = (conf->p_h_res()-images_per_row*image_width_all)/2;
  int x_start = x;
  int y = y_start;

  if (search_mode) {
    if (conf->p_input() != "keyboard") {
      y += 65;
      if (conf->p_h_res() == 800)
	y += 25;
    } else
      y += 45;
  }

  for (int j = 0; j < cur_rows*images_per_row && j < cur_list.size(); ++j) {

    Simplefile s = vector_lookup(cur_list, start);

    if (s.type == "dir") {

      PObj *p;

      if (pos == j)
	p = new PObj(themes->pictures_dir_marked, x, y, 3, NOSCALING);
      else
	p = new PObj(themes->pictures_dir, x, y, 3, NOSCALING);

      p->x = p->real_x = x + (image_width_all - p->w)/2;
      p->y = p->real_y = y + (image_height_all_eks_text - p->h)/2;

      render->current.add(p);

      S_Touch::get_instance()->register_area(TouchArea(rect(p->x, p->y, p->w, p->h), p->layer, boost::bind(&Pictures::find_element_and_do_action, this, s)));

    } else { // file

      bool border = false;

      if (pos == j)
	border = true;

      render->create_scaled_image_wrapper(s.path, image_width, image_height);
      PFObj *p = new PFObj(s.path, x, y, image_width, image_height, border, 3);

      p->x = p->real_x = x + (image_width_all - (p->w + 10))/2;
      p->y = p->real_y = y + (image_height_all_eks_text - (p->h + 10))/2;

      // compensate for border following the new x and y calculations
      if (p->border) {
	p->real_x -= 5;
	p->real_y -= 5;
      } else {
	p->real_x -= 1;
	p->real_y -= 1;
      }

      render->current.add(p);

      S_Touch::get_instance()->register_area(TouchArea(rect(p->x, p->y, p->w, p->h), p->layer, boost::bind(&Pictures::find_element_and_do_action, this, s)));

    }

    string name = s.name;
    string old_name = name;

    string_format::format_to_size(name, "Vera/13", image_width_all-10, false);

    string name_second_part = old_name.substr(name.size());

    if (name_second_part.size() > 0 && name_second_part[0] != ' ') {
      // splitting words
      string::size_type last_whole_word = name.rfind(' ');
      if (last_whole_word != string::npos) {
	name = name.substr(0, last_whole_word);
	name_second_part = old_name.substr(name.size());
      }
    }

    int x_size = string_format::calculate_string_width(name, "Vera/13");

    string_format::format_to_size(name_second_part, "Vera/13", image_width_all-10, false);

    int x_size2 = string_format::calculate_string_width(name_second_part, "Vera/13");

    if (pos == j) {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text,
				   themes->pictures_marked_font1, themes->pictures_marked_font2,
				   themes->pictures_marked_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20,
				   themes->pictures_marked_font1, themes->pictures_marked_font2,
				   themes->pictures_marked_font3, 3));
    } else {
      render->current.add(new TObj(name, "Vera/13", x + (image_width_all-x_size)/2, y+image_height_all_eks_text,
				   themes->pictures_font1, themes->pictures_font2,
				   themes->pictures_font3, 3));
      render->current.add(new TObj(name_second_part, "Vera/13", x + (image_width_all-x_size2)/2, y+image_height_all_eks_text+20,
				   themes->pictures_font1, themes->pictures_font2,
				   themes->pictures_font3, 3));
    }

    x += image_width_all;

    if (start == cur_list.size()-1) {
      if ((j+1)%images_per_row != 0)
	j += (images_per_row-(j+1)%images_per_row);

      x = x_start;
      y += cur_image_height_all;

      start = 0;
      continue;

    } else
      ++start;

    if ((j+1)%images_per_row == 0) {
      y += cur_image_height_all;
      x = x_start;
    }
  }

  std::ostringstream position;

  if (search_mode)
    if (cur_list.size() > 0)
      position << pos_in_list + 1 << "/" << cur_list.size();
    else
      position << "";
  else
    position << folders.top().second+1 << "/" << cur_list.size();

  int x_size = string_format::calculate_string_width(position.str(), "Vera/18");

  render->current.add(new TObj(position.str(), "Vera/18", conf->p_h_res()-(60+x_size), 20,
			       themes->pictures_font1, themes->pictures_font2,
			       themes->pictures_font3, 3));

  render->draw_and_release("pictures");
}

void Pictures::print_fullscreen(const Simplefile& e, const string& left_path, const string& right_path)
{
  // sanity
  if (e.type == "dir")
    return;

  if (conf->p_priority_change())
    nice(19);

  render->prepare_new_image();

  render->current.add(new RObj(0, 0, conf->p_h_res(), conf->p_v_res(), 0, 0, 0, 255, 0));

  // FIXME: create thumbnail for at least the db stored rotation

  if (zoom_level != 1 || orientation != 0) {
    render->current.add(new PFSObj(e.path, conv::stob(opts.zoom()), orientation,
				   zoom_level, zoom_position, max_pos_v, max_pos_h, 1));
  } else {
    // use thumbnailed for performance
    render->create_scaled_image_wrapper(e.path, conf->p_h_res(), conf->p_v_res());

    render->current.add(new PFSObj(render->image_get_fullpath(e.path, conf->p_h_res(), conf->p_v_res()),
				   conv::stob(opts.zoom()), orientation, zoom_level, zoom_position,
				   max_pos_v, max_pos_h, 1));
  }

  if (conv::stob(opts.show_preview()) && zoom_level == 1) {

    // make sure the thumbnails are there
    render->create_scaled_image_wrapper(left_path, image_width, image_height);
    render->create_scaled_image_wrapper(right_path, image_width, image_height);

    bool is_transparent;

    // left
    std::pair<int, int> dimensions = render->image_dimensions(render->image_get_fullpath(left_path, image_width, image_height), &is_transparent);

    PFObj *left = new PFObj(left_path, 5, conf->p_v_res()-dimensions.first-5, image_width, image_height, true, 2);
    render->current.add(left);

    S_Touch::get_instance()->register_area(TouchArea(rect(left->x, left->y, left->w, left->h), left->layer, boost::bind(&Pictures::fullscreen_left, this)));

    // right
    dimensions = render->image_dimensions(render->image_get_fullpath(right_path, image_width, image_height), &is_transparent);

    PFObj *right = new PFObj(right_path, conf->p_h_res()-dimensions.second-5, conf->p_v_res()-dimensions.first-5, image_width, image_height, true, 2);
    render->current.add(right);

    S_Touch::get_instance()->register_area(TouchArea(rect(right->x, right->y, right->w, right->h), right->layer, boost::bind(&Pictures::fullscreen_right, this)));
  }

  std::ostringstream position;
  if (conv::stob(opts.recurse_in_fullscreen()))
    position << pos_recursion+1 << "/" << recurse_files.size();
  else
    position << folders.top().second+1 << "/" << pic_list.size();

  int x_size = string_format::calculate_string_width(position.str(), "Vera/18");

  int x_pos = 20, y_pos = 20;

  if (conf->p_busy_indicator()) {
    x_pos = 35;
    y_pos = 10;
  }

  render->current.add(new TObj(position.str(), "Vera/18", conf->p_h_res()-(x_pos+x_size), y_pos,
			       themes->pictures_font1, themes->pictures_font2,
			       themes->pictures_font3, 2));

  string name = e.name;

  if (pause && conv::stob(opts.slideshow()))
    name += dgettext("mms-pictures", " ( paused )");

  string_format::format_to_size(name, "Vera/16", conf->p_h_res()-(20+x_size+30), false);

  render->current.add(new TObj(name, "Vera/16", 20, 20,
			       themes->pictures_font1, themes->pictures_font2,
			       themes->pictures_font3, 2));

  render->draw_and_release("pictures full");

  if (conf->p_priority_change())
    nice(-19);
}

void Pictures::play_pictures_cd()
{
  navigating_media = true;

  top_media_folders.clear();
  top_media_folders.push_back(cd->get_mount_point());

  if (cd->data_disc()) {
    // backup ;)
    vector<Picture> old_list = pic_list;
    std::stack<std::pair<list<string>, int > > temp_folders = folders;
    while (!folders.empty())
      folders.pop();
    list<string> templs; templs.push_back(cd->get_mount_point());
    folders.push(std::make_pair(templs, 0));

    //mount the disc
    run::external_program("mount '" + cd->get_mount_point() + "'");

    bool valid_files = testdir(cd->get_mount_point());

    pic_list = rdir(cd->get_mount_point());

    if (pic_list.size() == 0 || !valid_files)
      Print pdialog(dgettext("mms-pictures", "The disc does not contain any recognizable files"),
		    Print::SCREEN);
    else
      mainloop();

    //umount the disc
    run::external_program("umount '" + cd->get_mount_point() + "'");

    pic_list = old_list;
    folders = temp_folders;
  } else
    Print pdialog(dgettext("mms-pictures", "The disc does not contain any recognizable files"),
		  Print::SCREEN);

  navigating_media = false;
}

// ====================================================================
//
// dir
//
// ====================================================================

bool Pictures::testdir(const string& dir)
{
  libfs_set_folders_first(opts.dir_order());

  for (file_iterator<file_t, default_order> i (dir); i != i.end (); ++i) {
    if (global->check_stop_bit())
      break;

    if (check_type((*i).getName(), conf->p_filetypes_p()) != emptyMyPair)
      return true;
  }

  return false;
}

void Pictures::insert_picture_into_db(const string& filename, const string& name)
{
  db_mutex.enterMutex();

  SQLQuery *q = db.query("Pictures", ("SELECT * FROM %t WHERE filename='" + string_format::escape_db_string(filename) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    delete q;
    db_mutex.leaveMutex();
    return; // file already exists, don't insert
  } else
    delete q;

  struct stat s;
  stat(filename.c_str(), &s);

  // FIXME: group id (value 6)
  char *tmp = sqlite3_mprintf("INSERT INTO Pictures VALUES(NULL, '%q', '%q', '%q', '%i', 0, 0, 0)",
			      filename.c_str(), name.c_str(), "", s.st_mtime);
  db.execute(tmp);
  sqlite3_free(tmp);

  db_mutex.leaveMutex();
}

// Add an picture to the list
Picture Pictures::addfile(const string& name, const MyPair& filetype)
{
  Picture p = addsimplefile(name, filetype);

  insert_picture_into_db(name, p.name);

  return p;
}

void Pictures::insert_file_into_db(const string& filename, const string& parent)
{
  bool is_dir = isDirectory(filename);

  string good_filename;

  if (is_dir)
    good_filename = string_format::unique_folder_name(filename);
  else
    good_filename = filename;

  db_mutex.enterMutex();

  // check if already exists
  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_db_string(good_filename) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    delete q;
    db_mutex.leaveMutex();
    return;
  } else
    delete q;

  string parent_id = get_parent_id(parent, db, db_mutex, navigating_media ? top_media_folders : picture_folders);

  string::size_type i;
  if ((i = filename.rfind('/')) == string::npos)
    i = 0;
  string name = filename.substr((i != 0) ? i+1 : i);

  if (!is_dir && (i = name.rfind('.')) != string::npos)
    name = name.substr(0, i);

  db_mutex.enterMutex();

  char *tmp_insert = sqlite3_mprintf("INSERT INTO Folders VALUES(NULL, '%q', '%q', '%q', '%q', '%q')",
				     parent_id.c_str(), good_filename.c_str(), name.c_str(),
				     string_format::lowercase(name).c_str(), conv::itos(is_dir).c_str());
  db.execute(tmp_insert);
  sqlite3_free(tmp_insert);

  db_mutex.leaveMutex();
}

int Pictures::db_id(const string& file, bool is_dir)
{
  int i = -1;

  string good_file;

  if (is_dir)
    good_file = string_format::unique_folder_name(file);
  else
    good_file = file;

  db_mutex.enterMutex();

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_db_string(good_file) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    SQLRow &row = (*q)[0];
    i = conv::atoi(row["id"]);
  }
  delete q;

  db_mutex.leaveMutex();

  return i;
}

bool Pictures::rdir_internal(const string& original_name, const string& argv,
			     vector<Picture> &cur_files, bool file_in_db)
{
  if (global->check_stop_bit()) {
    cur_files.clear();
    return false;
  }

  string cur_file = original_name;
  if (cur_file[cur_file.size()-1] == '/')
    cur_file = cur_file.substr(0, cur_file.size()-1);

  string name;
  if (argv[argv.size()-1] != '/')
    name = cur_file.substr(argv.size()+1);
  else
    name = cur_file.substr(argv.size());

  if (isDirectory (cur_file)) {
    // exclude folders ?
    if (pic_conf->p_excluded_picture_dirs().size() > 0) {
      list<string> excluded = pic_conf->p_excluded_picture_dirs();
      bool stop = false;

      foreach (string& excluded_dir, excluded)
	if (cur_file.substr(cur_file.rfind("/")+1) == excluded_dir) {
	  stop = true;
	  break;
	}

      if (stop)
	return true;
    }

    // dir
    Picture r;
    r.id = ++id;
    r.name = name;
    if (conf->p_convert())
      r.name = string_format::convert(r.name);
    r.lowercase_name = r.name;
    r.path = cur_file;
    r.type="dir";
    if (file_in_db)
      r.db_id = db_id(r.path, true);

    cur_files.push_back(r);

  } else {
    // file
    const MyPair filetype = check_type(cur_file, conf->p_filetypes_p());
    if (filetype != emptyMyPair)
      cur_files.push_back(get_file_from_path(cur_file, filetype, file_in_db));

    ++S_ScreenUpdater::get_instance()->status_progressbar;
  }

  return true;
}

Picture Pictures::get_file_from_path(const string& path, MyPair filetype, bool file_in_db)
{
  Picture p;
  if (in_fullscreen)
    p = addsimplefile(path, filetype);
  else
    p = addfile(path, filetype);
  if (file_in_db)
    p.db_id = db_id(p.path, false);
  return p;
}

vector<string> Pictures::check_db_for_folders(const string& parent)
{
  vector<string> names;

  string good_parent = string_format::unique_folder_name(parent);

  db_mutex.enterMutex();

  SQLQuery *q = db.query("Folders", ("SELECT id FROM %t WHERE filename='" + string_format::escape_db_string(good_parent) + "'").c_str());
  if (q && q->numberOfTuples() > 0) {
    assert(q->numberOfTuples() == 1);
    SQLRow &row = (*q)[0];
    string parent_id = row["id"];

    SQLQuery *q2;

    if (opts.dir_order() == dgettext("mms-pictures", "directories first"))
      q2 = db.query("Folders", ("SELECT filename, upper(name) as key FROM %t WHERE parent='" + parent_id + "' ORDER BY is_folder desc, key").c_str());
    else
      q2 = db.query("Folders", ("SELECT filename, upper(name) as key FROM %t WHERE parent='" + parent_id + "' ORDER BY key").c_str());

    if (q2) {
      for (int i = 0; i < q2->numberOfTuples(); ++i) {
	SQLRow &row = (*q2)[i];
	names.push_back(row["filename"]);
      }
    }
    delete q2;
  }
  delete q;

  db_mutex.leaveMutex();

  return names;
}

vector<Picture> Pictures::rdir_hd(const string& argv, bool insert_into_db)
{
  libfs_set_folders_first(opts.dir_order());

  if (insert_into_db) {

    bool inserted = false;

    for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false)) {
      string filename = filesystem::FExpand(i->getName());
      if (isDirectory(filename) || check_type(filename, conf->p_filetypes_p()) != emptyMyPair) {
	insert_file_into_db(filename, argv);
	inserted = true;
      }
    }

    if (inserted)
      return rdir(argv);
    else {
      vector<Picture> cur_files;
      return cur_files;
    }

  } else {

    vector<Picture> cur_files;

    for (file_iterator<file_t, default_order> i (argv); i != i.end (); i.advance(false)) {
      string filename = filesystem::FExpand(i->getName());
      if (!rdir_internal(filename, argv, cur_files, false))
	break;
    }

    return cur_files;
  }
}

// read a dir and spit out a list of files and dirs
vector<Picture> Pictures::rdir(const string& argv)
{
  vector<Picture> files;

  vector<string> db_files = check_db_for_folders(argv);

  if (db_files.size() > 0) {
    foreach (string& dir, db_files)
      if (!rdir_internal(dir, argv, files, true))
	break;
  } else {
    files = rdir_hd(argv, true);
  }

  return files;
}

// The ber function for parsing dirs.
vector<Picture> Pictures::parse_dir(const list<string>& dirs)
{
  vector<Picture> files;

  foreach (const string& dir, dirs) {
    vector<Picture> tempfiles = rdir(dir);
    files.insert(files.end(), tempfiles.begin(), tempfiles.end());
  }

  return files;
}

// functions for creating lists for recursion

vector<Picture> Pictures::parse_dirs_recursion()
{
  vector<string> image_files = images_in_dir(folders.top().first);

  // we need them in the correct order for recursion
  sort(image_files.begin(), image_files.end());

  vector<Picture> files;

  foreach (string& image, image_files)
    {
      const MyPair filetype = check_type(image, conf->p_filetypes_p());
      if (filetype != emptyMyPair)
	files.push_back(get_file_from_path(image, filetype));
    }

  return files;
}

void Pictures::read_dirs()
{
  // process picture dir
  pic_list = parse_dir(picture_folders);

  if (picture_folders.size() > 1)
    std::sort(pic_list.begin(), pic_list.end(), file_sort());

  if (pic_list.size() != 0)
    folders.top().second = 0;

  S_BackgroundUpdater::get_instance()->run_once(boost::bind(&Pictures::check_for_changes, this));
}

#ifdef use_inotify
// notify
void Pictures::fs_change(NotifyUpdate::notify_update_type type, const string& path)
{
  if (type == NotifyUpdate::MODIFY_FILE)
    ; // FIXME: doesn't work because inotify returns dir and not the file: reparse_file(path);
  else
    reload_dir(path);

  foreach (string& dir, folders.top().first)
    if (path == dir) {
      reparse_current_dir();
      break;
    }

  if (type == NotifyUpdate::DELETE_DIR) {
    bool current_dir_ok = false;

    while (!current_dir_ok) {
      load_current_dirs();
      if (pic_list.size() == 0) {
	if (pic_list.size() == 1) {
	  exit();
	  break;
	}
	else
	  folders.pop();
      } else
	current_dir_ok = true;
    }
  }

  // sanity check
  if (folders.top().second > pic_list.size()-1)
    folders.top().second = pic_list.size()-1;

  if (!global->playback_in_fullscreen && !in_fullscreen && visible)
    print(pic_list);
}
#endif

void Pictures::reparse_current_dir()
{
  // process picture dir
  reload_current_dirs();

  // sanity check
  if (folders.top().second > pic_list.size()-1)
    folders.top().second = 0;
}

void Pictures::pictures_check()
{
  if (opts.slideshow() != "0" && !pause && last_press <= time(0) - conv::atoi(opts.slideshow())) {

    if (conv::stob(opts.slideshow_random())) {
      int list_size = pic_list.size();
      if (conv::stob(opts.recurse_in_fullscreen()))
	list_size = recurse_files.size();
      new_random_fullscreen(list_size);
    } else {
      if (conv::stob(opts.recurse_in_fullscreen()))
	next_no_skip_folders();
      else
	next_skip_folders();
    }

    orientation = get_db_orientation_lookup();

    prepare_and_show_fullscreen();
  }
}

int Pictures::check_picture_time()
{
  // here we can assume that opts.slideshow() > 0 from the calling code
  return std::min(static_cast<long int>(conv::atoi(opts.slideshow())), time(0)-last_press)*1000;
}
