#include "Python.h"
#include <stdio.h>
#include <unistd.h>
#include "libfspp.hpp"
#include "config.hpp"
#include "guiwindowscripts.hpp"
#include "mmspython.hpp"
#include "guiwindowmanager.hpp"
#include "python_config.hpp"

#define GUILISTCONTROL 10

using std::string;
using std::vector;

GUIWindowScripts::GUIWindowScripts()
{}

GUIWindowScripts::~GUIWindowScripts()
{
  clear();
}

Simplefile GUIWindowScripts::addSimpleFile(const string& strName, 
				const MyPair& fileType, bool bFromMedia)
{
  Config* pConf = S_Config::get_instance();
  Simplefile r;
  r.path = strName;
  string strTempName = strName.substr(0, strName.size()-((fileType.first).size()+1));
  string::size_type pos = strTempName.rfind("/");

  if (pos != string::npos)
    r.name = strTempName.substr(pos + 1);
  else
    r.name = strTempName;

  if (pConf->p_convert())
    r.name = string_format::convert(r.name);

  if (bFromMedia)
    r.type = "media-file";
  else
    r.type = fileType.second;

  return r;
}

void GUIWindowScripts::readDir(const string& strArgv)
{
  Config* pConf = S_Config::get_instance();
  PythonConfig* pPythonConf = S_PythonConfig::get_instance();
  FileItem* pItem = NULL;

  GUIMessage resetMsg(GUI_MSG_LABEL_RESET, getId(), GUILISTCONTROL);
  onMessage(resetMsg);

  for (vector<FileItem*>::iterator i = m_vecFileItems.begin(),
          end = m_vecFileItems.end(); i != end; i++) {
      delete (*i);
  }  
  m_vecFileItems.clear();
  

  for (file_iterator<file_t, default_order> i (strArgv); i != i.end (); i.advance(false))
  {
    pItem = NULL;
    string strPath = filesystem::FExpand(i->getName());

    if (isDirectory (strPath))
    {
      // dir
      Simplefile r;
      r.name = strPath.substr(strArgv[strArgv.size()-1] == '/' ? 
				  strArgv.size() : strArgv.size() + 1);
      if (pConf->p_convert())
        r.name = string_format::convert(r.name);

      r.path = strPath;
      r.type = "dir";

      pItem = new FileItem(r);
      pItem->setThumbImage("python/dir.png");
      m_vecFileItems.push_back(pItem);
    } 
    else 
    {
      // file
      const MyPair fileType = check_type(strPath, pPythonConf->p_filetypes_s());

      if (fileType != emptyMyPair)
      {
	pItem = new FileItem(addSimpleFile(strPath, fileType));
        pItem->setThumbImage("python/exec.png");
	m_vecFileItems.push_back(pItem);
      }
    }

    if(pItem)
    {
      GUIMessage addMsg(GUI_MSG_LABEL_ADD, getId(),GUILISTCONTROL, 0, 0, (void*)pItem);
      onMessage(addMsg);
    } 
  }
}

bool GUIWindowScripts::onAction(const string& strAction)
{
  GUIWindow::onAction(strAction);

  if(strAction == "back")
  {
     if(m_vecHistoryDirs.size() > 1)
     {
        m_vecHistoryDirs.erase(m_vecHistoryDirs.begin());
        readDir(m_vecHistoryDirs.front());
     }
     else
	return false;
  }

  if(strAction == "action")
  {
     if(!m_vecFileItems.empty())
     {
       GUIMessage posMsg(GUI_MSG_ITEM_SELECTED, getId(), GUILISTCONTROL);
       onMessage(posMsg);

       if(m_vecFileItems[posMsg.getParam1()]->getFileInfo().type == "dir")
       { 
          m_vecHistoryDirs.insert(m_vecHistoryDirs.begin(), 
			   m_vecFileItems[posMsg.getParam1()]->getFileInfo().path);
          readDir(m_vecHistoryDirs.front());
       }
       else
       {
          pymms::MMSPython* pMmsPython = pymms::S_MMSPython::get_instance();
          pMmsPython->evalFile(m_vecFileItems[posMsg.getParam1()]->getFileInfo().path);
       }
     }						  	
  }

  S_WindowManager::get_instance()->render();

  return true;
}

bool GUIWindowScripts::load(const string& strFileName)
{
   PythonConfig* pPythonConf = S_PythonConfig::get_instance();
   bool bLoadable = false;

   if (file_exists(pPythonConf->p_script_dir()))
      bLoadable = GUIWindow::load(strFileName);
   else
     std::cerr << "WARNING could not load: " << strFileName << std::endl;
   
   if (bLoadable)
   {
      m_vecHistoryDirs.insert(m_vecHistoryDirs.begin(), pPythonConf->p_script_dir());
      readDir(m_vecHistoryDirs.front());
   }

   return bLoadable;
}
