#include <Python.h>
#include "structmember.h"
#include "pyplayer.hpp"
#include "pyplaylist.hpp"

using std::string;
using std::pair;
using std::vector;

#ifdef __cplusplus
extern "C" {
#endif

namespace pymms {

namespace python {

static PyObject* Player_New(PyTypeObject *type, PyObject *args, PyObject *kwds)
{
  Player *self;
  bool bPlayer = false;

  static char *kwlist[] = {"player", NULL};

  self = (Player*)type->tp_alloc(type, 0);
  if (!self)
    return NULL;

  self->pyPlayList = (PlayList*)PlayList_Type.tp_new(&PlayList_Type, args, kwds);
  if (!self->pyPlayList)
    return NULL;  

  if (!PyArg_ParseTupleAndKeywords(args, kwds, "|b", kwlist, &bPlayer))
    return NULL;
    
  if(bPlayer)
    self->pPlayer = S_PythonVideoPlayer::get_instance();
  else
    self->pPlayer = S_PythonAudioPlayer::get_instance();

  return (PyObject*)self;
}

static PyObject* Player_play(Player *self, PyObject *args)
{
  char *strTitle = "";
  PyObject* pObject = NULL;

  if (!PyArg_ParseTuple(args, "|Os", &pObject, &strTitle))	
    return NULL;

  if(PlayList_Check(pObject))
  {
    PlayList* pyPlayList = (PlayList*)pObject;
      
    self->pyPlayList->pPlayList->clear();
    self->pyPlayList->pPlayList->copy(*(pyPlayList->pPlayList));
  }
  else if(PyString_Check(pObject))
  {
    char* strPath = PyString_AsString(pObject);
    pair<string, string> item(strPath, strTitle);

    self->pyPlayList->pPlayList->clear();
    self->pyPlayList->pPlayList->add(item);
  }
  else if(PyInt_Check(pObject))
  {
    int iItem = PyInt_AsLong(pObject);  
    self->pyPlayList->pPlayList->set(iItem);
  }

  if(!self->pyPlayList->pPlayList->empty())
    self->pPlayer->play(self->pyPlayList->pPlayList->get().first, self->pyPlayList->pPlayList->get().second);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_stop(Player *self, PyObject *args)
{
  self->pPlayer->stop();

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_pause(Player *self, PyObject *args)
{
  self->pPlayer->pause();

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_mute(Player *self, PyObject *args)
{
  self->pPlayer->mute();

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_volup(Player *self, PyObject *args)
{
  self->pPlayer->volup();
  
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_voldown(Player *self, PyObject *args)
{
  self->pPlayer->voldown();

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_ff(Player *self, PyObject *args)
{
  self->pPlayer->ff();

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_fb(Player *self, PyObject *args)
{
  self->pPlayer->fb();

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_next(Player* self, PyObject* args)
{
  self->pyPlayList->pPlayList->next();

  if(!self->pyPlayList->pPlayList->empty())
    self->pPlayer->play(self->pyPlayList->pPlayList->get().first, self->pyPlayList->pPlayList->get().second);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_prev(Player* self, PyObject* args)
{
  self->pyPlayList->pPlayList->prev();

  if(!self->pyPlayList->pPlayList->empty())
    self->pPlayer->play(self->pyPlayList->pPlayList->get().first, self->pyPlayList->pPlayList->get().second);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* Player_getPlayList(Player *self, PyObject *args)
{
  Py_INCREF(self->pyPlayList);

  return (PyObject*)self->pyPlayList;
}

static void Player_Dealloc(Player* self)
{
  Py_DECREF(self->pyPlayList);
  self->ob_type->tp_free((PyObject*)self);
}

static PyMethodDef Player_methods[] = {
  {"play", (PyCFunction)Player_play, METH_VARARGS, 0},
  {"stop", (PyCFunction)Player_stop, METH_VARARGS, 0},
  {"pause", (PyCFunction)Player_pause, METH_VARARGS, 0},
  {"mute", (PyCFunction)Player_mute, METH_VARARGS, 0},
  {"volup", (PyCFunction)Player_volup, METH_VARARGS, 0},
  {"voldown", (PyCFunction)Player_voldown, METH_VARARGS, 0},
  {"ff", (PyCFunction)Player_ff, METH_VARARGS, 0},
  {"fb", (PyCFunction)Player_fb, METH_VARARGS, 0},
  {"next", (PyCFunction)Player_next, METH_VARARGS, 0},
  {"prev", (PyCFunction)Player_prev, METH_VARARGS, 0},
  {"getPlayList", (PyCFunction)Player_getPlayList, METH_VARARGS, 0},
  {NULL, NULL, 0, NULL}
};

PyTypeObject Player_Type = {
	PyObject_HEAD_INIT(NULL)
	0,                         /*ob_size*/
	"mmsv2gui.Player",    /*tp_name*/
	sizeof(Player),      /*tp_basicsize*/
	0,                         /*tp_itemsize*/
	(destructor)Player_Dealloc, 	/*tp_dealloc*/
	0,                         /*tp_print*/
	0,                         /*tp_getattr*/
	0,                         /*tp_setattr*/
	0,                         /*tp_compare*/
	0,                         /*tp_repr*/
	0,                         /*tp_as_number*/
	0,                         /*tp_as_sequence*/
	0,                         /*tp_as_mapping*/
	0,                         /*tp_hash */
	0,                         /*tp_call*/
	0,                         /*tp_str*/
	0,                         /*tp_getattro*/
	0,                         /*tp_setattro*/
	0,                         /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
	0,       /* tp_doc */
	0,		                     /* tp_traverse */
	0,		                     /* tp_clear */
	0,		                     /* tp_richcompare */
	0,		                     /* tp_weaklistoffset */
	0,		                     /* tp_iter */
	0,		                     /* tp_iternext */
	Player_methods,      /* tp_methods */
	0,                         /* tp_members */
	0,                         /* tp_getset */
	0,             /* tp_base */
	0,                         /* tp_dict */
	0,                         /* tp_descr_get */
	0,                         /* tp_descr_set */
	0,                         /* tp_dictoffset */
	0,                         /* tp_init */
	0,                         /* tp_alloc */
	Player_New,          /* tp_new */
};

}

}

#ifdef __cplusplus
}
#endif
