#include "videoplayer.hpp"
#include "config.h"
#include "plugins.hpp"
#include "movie.hpp"

using std::string;

pthread_mutex_t pymms::player::PythonVideoPlayer::singleton_mutex = PTHREAD_MUTEX_INITIALIZER;

pymms::player::PythonVideoPlayer::PythonVideoPlayer()
{
  MoviePlayerPlugin *pPlugin = 0;
  m_bSet = false;
  Plugins *pPlugins = S_Plugins::get_instance();
  Movie* pMovie = get_class<Movie>(dgettext("mms-movie", "Movie"));

  pPlugin = pPlugins->find(pPlugins->movie_players, pMovie->get_opts()->movie_player());
  if(pPlugin)
  {
    m_pPlayer = pPlugin->player;
    m_bSet = true;
  }

  if(!m_bSet)
    DialogWaitPrint pdialog(dgettext("mms-python", "No movie support detected"), 2000);

}

void pymms::player::PythonVideoPlayer::play(string strPath, string strTitle)
{
  play(strPath, true);
}
    
void pymms::player::PythonVideoPlayer::play(string strPath, bool bWindow)
{
  #ifdef c_movie
  if(m_bSet)
  {
    // stop audio playback if needed
    if (S_Global::get_instance()->is_playback("Audio")) {
      Input stop;
      stop.command = "stop";
      stop.mode = "playback";

      S_Global::get_instance()->check_commands(stop);
      S_Global::get_instance()->set_playback("Movie");
    }

    if (m_pPlayer->wants_exclusivity())
      S_Render::get_instance()->device->unlock();

    S_BusyIndicator::get_instance()->disable();

    m_pPlayer->play_movie(strPath, bWindow);

    if (m_pPlayer->wants_exclusivity())
      S_Render::get_instance()->device->lock();
    else
      S_Render::get_instance()->complete_redraw();

    S_BusyIndicator::get_instance()->enable();
  }
  #endif
}

void pymms::player::PythonVideoPlayer::pause()
{
  #ifdef c_movie
  if(m_bSet)
  {
    m_pPlayer->pause();
  }
  #endif
}

void pymms::player::PythonVideoPlayer::stop()
{
  #ifdef c_movie
  if(m_bSet)
  {
    m_pPlayer->stop();
  }
  #endif
}

void pymms::player::PythonVideoPlayer::ff()
{
  #ifdef c_movie
  if(m_bSet)
  {
    m_pPlayer->ff();
  }
  #endif
}

void pymms::player::PythonVideoPlayer::fb()
{
  #ifdef c_movie
  if(m_bSet)
  {
    m_pPlayer->fb();
  }
  #endif
}
