
#include "config.h"
#include "weather.hpp"
#include "weather_config.hpp"
#include "gettext.hpp"
#include "graphics.hpp"
#include "busy_indicator.hpp"

#include "startmenu.hpp"
#include "updater.hpp"
#include "common-feature.hpp"
#include "tinyxml.h"
#include <string>
#include <sstream>

#ifdef use_nls
#include <langinfo.h>
#include <locale.h>
#endif
using namespace std;

string CData::get_name ( int dindex ) {
   if ( day[ dindex ].name == "Today" )  
      day[ dindex ].name = dgettext("mms-weather","Today");
   else if ( day[ dindex ].name == "Tomorrow" ) 
      day[ dindex ].name = dgettext("mms-weather","Tomorrow");
   else if ( day[ dindex ].name == "Monday" ) 
      day[ dindex ].name = dgettext("mms-weather","Monday");
   else if ( day[ dindex ].name == "Tuesday" ) 
      day[ dindex ].name = dgettext("mms-weather","Tuesday");
   else if ( day[ dindex ].name == "Wednesday" ) 
      day[ dindex ].name = dgettext("mms-weather","Wednesday");
   else if ( day[ dindex ].name == "Thursday" ) 
      day[ dindex ].name = dgettext("mms-weather","Thursday");
   else if ( day[ dindex ].name == "Friday" ) 
      day[ dindex ].name = dgettext("mms-weather","Friday");
   else if ( day[ dindex ].name == "Saturday" ) 
      day[ dindex ].name = dgettext("mms-weather","Saturday");
   else if ( day[ dindex ].name == "Sunday" ) 
      day[ dindex ].name = dgettext("mms-weather","Sunday");
    
   return (string) day[ dindex ].name;
}

int CData::get_code ( int dindex ) {
   return day[ dindex ].code;
}

int CData::get_high_temp ( int dindex ) {
   return day[ dindex ].hi;
}

int CData::get_low_temp ( int dindex ) {
   return day[ dindex ].low;
}

string CData::get_city ( void ) {
   return city;
}

string CData::get_country ( void ) {
   return country;
}

CData::CData()
{}

CData::CData ( string xml_filename, string city_code, string data_file ,string data_unit)
{
  int iter=0;
   /*
  printf("weather: Using <%s> for updating weather information.\n", xml_filename.c_str() );
  printf("weather: Using <%s> as the code for your city.\n", city_code.c_str() );
  printf("weather: Using <%s> for file where information is stored.\n", data_file.c_str() );
  printf("weather: Using <%c> as unit (c=Celsius; f=Fahrenheit).\n", data_unit.c_str() );
*/
  TiXmlDocument doc(data_file.c_str());
  TiXmlElement* root_element;
  TiXmlElement* element;
  bool loadOkay = doc.LoadFile();
  if (loadOkay)
    {
      //      printf("\n%s:\n", data_file.c_str());
      //dump_to_stdout( &doc ); // defined later in the tutorial
    }
  else
    {
      cout << "weather: DEBUG: Failed to load file " <<  data_file << endl;
      day[0].name  = "missing";
      return;
    }

  root_element = doc.RootElement();
  day[0].name  = "N/A";
  if (root_element != 0)
  {
     TiXmlHandle myhandle(&doc);
     element=myhandle.FirstChild("weather").FirstChild("location").Element();
     city=element->Attribute("city");
     country=element->Attribute("country");
     element = myhandle.FirstChild("weather").FirstChild("location").FirstChild("forecast").FirstChild("day").Element();
     if ( element == NULL ) {
       DialogWaitPrint pdialog(3000);
       pdialog.add_line(dgettext("mms-weather", "Weather information"));
       pdialog.add_line(dgettext("mms-weather", "currently not available."));
       pdialog.add_line("");
       pdialog.add_line(dgettext("mms-weather", "Please try again later."));
       pdialog.print();
     } else {
     for ( element; element ; element=element->NextSiblingElement("day") )
       {
         day[iter].name   = (string)element->Attribute("name");
         element->FirstChildElement("temp")->QueryIntAttribute("high", &day[iter].hi);
         element->FirstChildElement("temp")->QueryIntAttribute("low",  &day[iter].low);
         element->FirstChildElement("condition")->QueryIntAttribute("code", &day[iter].code);
         iter++;
         /*
         //Gutes Bsp, um an den Text zwischen den Tags zu kommen
         TiXmlHandle myhandle(child);
         TiXmlText* text = myhandle.FirstChildElement("hi").FirstChild().Text();
         const char* message = text->Value();
         printf("weather: DEBUG: Temperature: hi:<%s>\n",  message );
         */
       }
     }
   }
}

Weather::Weather() : Unit("ºC"), enabled(false)
{
#ifdef use_nls
   setlocale(LC_ALL, ""); /* set from the environment variables */
   bindtextdomain("mms-weather", mms_prefix"/share/locale");
   bind_textdomain_codeset("mms-weather", nl_langinfo(CODESET));
#endif
   curlayer = 0;

#ifdef use_notify_area
   nArea->add(NotifyElement("Weather", boost::bind(&Weather::ShowNotifyArea, this))); 
#endif

   WeatherConfig *weather_conf = S_WeatherConfig::get_instance();
   if ( weather_conf->p_weather_unit() != "c" ) 
     Unit="°F";
   else 
     Unit="°C";
  
  update_wdata();

  ScreenUpdater *screen_updater = S_ScreenUpdater::get_instance();
  screen_updater->timer.add(TimeElement("weather update", 
	boost::bind(&Weather::update_idle, this),
        boost::bind(&Weather::update_wdata, this)));
}

int Weather::update_idle()
{
  return 1000 * 15;
}

void Weather::update_wdata()
{
  WeatherConfig *weather_conf = S_WeatherConfig::get_instance();
  wdata = CData (weather_conf->p_weather_update_script(),
          weather_conf->p_weather_city_code(),
          weather_conf->p_weather_data(),
          weather_conf->p_weather_unit()   
  );
  if (wdata.get_name(0) == "N/A" || wdata.get_name(0) == "missing")
	enabled = false;
  else
	enabled = true; 
}

std::string Weather::Print()
{
  
  //if (!file_exists(weather_conf->p_weather_data())) {
  if (!enabled) {

    DialogWaitPrint pdialog(3000);

    pdialog.add_line(dgettext("mms-weather", "Error downloading weather data"));
    pdialog.add_line("");
    pdialog.add_line(dgettext("mms-weather", "Please check your connection"));
    pdialog.add_line(dgettext("mms-weather", "and/or configuration file"));
    pdialog.print();

    return "failed";
  }

  render->prepare_new_image();

  render->current.add(new PObj(themes->weather_background, 0, 0, 0, SCALE_FULL));
  render->current.add(new PObj(themes->startmenu_weather, 25, 10, 1, NOSCALING));
  render->current.add(new TObj(dgettext("mms-weather","Weather"), "Vera/28", 100, 5,
                themes->weather_header_font1, themes->weather_header_font2,
                themes->weather_header_font3, 2));

  // Go and get weather data
  int dx_pos=conf->p_h_res() / 6;
  render->current.add(new TObj( wdata.get_city() + ", " +  wdata.get_country(),
            "Vera/18", dx_pos/2, 70,
            themes->weather_header_font1,
            themes->weather_header_font2,
            themes->weather_header_font3, 2) );
     for ( int iicon=0; iicon <5; iicon ++ ) {

       /* horizontal icon position of icons on screen */
       int x_pos=dx_pos*iicon + (dx_pos/2)+16;

       render->current.add(new PObj("./weather/large_icons/" + conv::itos(wdata.get_code(iicon)) + ".png", x_pos, 170, 1, NOSCALING));

       render->current.add(new TObj( wdata.get_name(iicon),
                 "Vera/18", x_pos + 10, 120,
                 themes->weather_header_font1,
                 themes->weather_header_font2,
                 themes->weather_header_font3, 2) );

       int max_width = std::max(string_format::calculate_string_width(dgettext("mms-weather","High: "), "Vera/16"),
				string_format::calculate_string_width(dgettext("mms-weather","Low: "), "Vera/16"));

       int max_unit_width = std::max(string_format::calculate_string_width(conv::itos(wdata.get_high_temp(iicon)) + Unit, "Vera/16"),
				     string_format::calculate_string_width(conv::itos(wdata.get_low_temp(iicon)) + Unit, "Vera/16"));

       render->current.add(new TObj(dgettext("mms-weather","High: "),
                 "Vera/16", x_pos + 10, 280,
                 themes->weather_header_font1,
                 themes->weather_header_font2,
                 themes->weather_header_font3, 2) );

       int right_aligned_offset = max_unit_width - string_format::calculate_string_width(conv::itos(wdata.get_high_temp(iicon)) + Unit, "Vera/16");

       render->current.add(new TObj(conv::itos(wdata.get_high_temp(iicon)) + Unit,
                 "Vera/16", x_pos + 10 + max_width + right_aligned_offset, 280,
                 themes->weather_header_font1,
                 themes->weather_header_font2,
                 themes->weather_header_font3, 2) );

       render->current.add(new TObj(dgettext("mms-weather","Low: "),
                 "Vera/16", x_pos + 10, 320,
                 themes->weather_header_font1,
                 themes->weather_header_font2,
                 themes->weather_header_font3, 2) );

       right_aligned_offset = max_unit_width - string_format::calculate_string_width(conv::itos(wdata.get_low_temp(iicon)) + Unit, "Vera/16");

       render->current.add(new TObj(conv::itos(wdata.get_low_temp(iicon)) + Unit,
				    "Vera/16", x_pos + 10 + max_width + right_aligned_offset, 320,
				    themes->weather_header_font1,
				    themes->weather_header_font2,
				    themes->weather_header_font3, 2) );
   }

  render->device->animation_section_begin();
  render->device->switch_to_layer(curlayer+1);
  render->device->reset_layout_attribs_nowait();
  render->device->animation_zoom(0.90,0.90,1,1,40,curlayer+1);
  render->device->animation_fade(0,1,50,curlayer+1);
  render->device->animation_fade(1,0,60,curlayer);
  render->device->animation_section_end();

  render->draw_and_release("mms-weather");

  return "";
}

std::string Weather::mainloop()
{
  bool exit_loop=false;
  bool update_needed=false;
  bool failed=false;

  BusyIndicator *busy_indicator = S_BusyIndicator::get_instance();
  WeatherConfig *weather_conf = S_WeatherConfig::get_instance();
  run::external_program(     weather_conf->p_weather_update_script() + " "
                           + weather_conf->p_weather_city_code() + " "
                           + weather_conf->p_weather_data() + " "
                           + weather_conf->p_weather_unit() );

   cout << "weather: executing " << weather_conf->p_weather_update_script() << endl;
  render->device->animation_section_begin();
  curlayer = render->device->get_current_layer();
  render->device->animation_section_end();
  if (Print() == "failed")
    return "";

  Input input;

  // main input loop
  while (!exit_loop) {
    if (update_needed){
      update_needed=false;

      render->device->animation_section_begin();
      render->device->swap_layers(curlayer, curlayer+1);
      render->device->animation_section_end();
      failed =  ( Print() == "failed" );
    }
    //else 
      input = input_master->device->get_input_busy_wrapped(busy_indicator);
    
    if (input.command == "back" || failed == true )
    {
        exit_loop=true;
    }
    else
      update_needed=!global->check_commands(input);
  }
  render->device->animation_section_begin();
  render->device->switch_to_layer(curlayer);
  render->device->reset_layout_attribs_nowait();
  render->device->animation_fade(0,1,80,curlayer);
  render->device->animation_zoom(1,1,4,4,40,curlayer+1);
  render->device->animation_fade(1,0,40,curlayer+1);
  render->device->animation_section_end();
  return "";
}

#ifdef use_notify_area
void Weather::ShowNotifyArea() 
{ 
  string strMsg;
  std::pair<int, int> size;
  int x, y;   
  int iicon = 0;
  strMsg = conv::itos(wdata.get_low_temp(iicon)) + Unit + " / " + conv::itos(wdata.get_high_temp(iicon)) + Unit;
      size = string_format::calculate_string_size(strMsg, "Vera/12");
      nArea->status_overlay.add(new TObj(strMsg, "Vera/12", 
			conf->p_h_res() - NotifyArea::width/2 - size.first/2,
			conf->p_v_res() - size.second - 2, 
			themes->notify_area_font1, 
			themes->notify_area_font2, 
			themes->notify_area_font3, 3));

      PObj *icon = new PObj("./weather/middle_icons/" + conv::itos(wdata.get_code(iicon)) + ".png", x, conf->p_v_res() - size.second, 3, NOSCALING);
      icon->x = icon->real_x = conf->p_h_res() - NotifyArea::width/2 - icon->w/2;
      icon->y = icon->real_y = conf->p_v_res() - size.second - icon->h;
      
      nArea->status_overlay.add(icon);

}
#endif
