#ifndef INPUT_HPP
#define INPUT_HPP

#include "config.h"

#include "common.hpp"
#include "config.hpp"
#include "updater.hpp"
#include "extra_menu.hpp"
#include "touch.hpp"

#include "singleton.hpp"

#include "boost.hpp"

#include <cc++/thread.h>

#include "libfspp.hpp"

#include <string>
#include <map>
#include <vector>

#include <iostream>

// structure used for input keys
class Input
{
public:
  std::string mode, key, command, presentation, name;
  bool multiple;

  bool operator==(const Input& i) const
  {
    return mode == i.mode && command == i.command;
  }

  std::string to_string() const;
  // create from string
  Input(std::string& s);
  Input(const std::string& command, const std::string mode);

  Input();
};

typedef std::map<std::string, Input> input_map;
typedef std::vector<Input> input_list;

typedef std::pair<std::string, std::pair<input_map, input_list> > name_input;
typedef std::map<std::string, std::pair<input_map, input_list> > name_input_map;

class InputDevice;

class InputMaster
{
private:

  std::pair<bool, std::pair<input_map, input_list> > parse_config(const std::string& plugin,
								  const std::string& version, bool extends);
  bool check_version(const std::string& input, const std::string& str, const std::string& version);

  std::string saved_map, cur_map;
  std::string input_device_name;
  bool process_input;
public:

  static pthread_mutex_t singleton_mutex;
  void input_device_loaded();
  std::string get_device_name(){
    if (process_input)
      return input_device_name;
    else
      return "Invalid";
  }

  std::string find_shortcut(const std::string& command);
  input_list cur_list();

  name_input_map maps;

  bool restore_map_empty();
  void save_map();
  void restore_map();
  void set_map(const std::string& name);
  std::string current_map();
  std::string current_saved_map();

  bool find_input(const std::string& input, Input &i);

  InputDevice *device;

  // const input_desc_map& desc,
  bool parse_keys(const std::string& plugin, const std::string& version);
  bool extend_default(const std::string& plugin, const std::string& version);

  InputMaster();
};

typedef Singleton<InputMaster> S_InputMaster;

#include "input_device.hpp"

#endif
