#ifndef SDLOUT_HPP
#define SDLOUT_HPP

#include "render_device.hpp"
#include "renderer.hpp"

#include <SDL/SDL.h>
#include <SDL/SDL_syswm.h>

class SDLdev : public RenderDevice
{
private:
  int bpp;

  int sdl_window_id;

  int width, height;
  int line_len;

  int initialized;
  int verbose;

  bool fullscreen;

  int window_id;
  bool window_mapped;

  int fs_window_id;
  bool fs_window_mapped;

#ifdef use_x11
  Display *display;
#endif
  SDL_Surface *screen;
  SDL_Surface *rgbsurface;

  ost::Mutex locked_mutex;

  void draw_rectangle(int x, int y, int w, int h);

  void sdldev_draw();

  void sdldev_draw_32();

  Render *render;

public:

  ost::Event get_input_event;
  std::string input_key;

  SDLdev();
  ~SDLdev();

  void init();
  void deinit();

  void lock();
  void unlock();

  bool supports_wid();

  int wid();
  int fs_wid();

  void show_window_video();
  void hide_window_video();

  void show_fs_video();
  void hide_fs_video();

  void hide_video();

  void toggle_fullscreen(bool locked, bool leave_with_lock_open);

 private:
  void run();
};

#endif
